@@ -4,7 +4,7 @@
  *   Copyright (C) 2004, 2005, 2006, 2007 by Stephen McInerney             *
  *   spm@stedee.id.au                                                      *
  *                                                                         *
- *   $Id: common.h 350 2007-01-19 09:20:00Z steve $
+ *   $Id: common.h 406 2007-05-23 12:20:04Z steve $
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -149,5 +149,12 @@
 #define CLK_TCK _SC_CLK_TCK
 #endif          /* CLK_TCK */
 
+/* Bring in linux kernel string.c code */
+#ifndef HAVE_STRLCPY
+size_t strlcpy(char *, const char *, size_t);
+#endif          /* HAVE_STRLCPY */
+
+
+
 #endif          /* AWFFULL_COMMON_H */
 /* END OF FILE */
