@@ -1,7 +1,7 @@
 /*
     AWFFull - A Webalizer Fork, Full o' features
     
-    $Id: css_file.c 350 2007-01-19 09:20:00Z steve $
+    $Id: css_file.c 367 2007-02-11 10:27:18Z steve $
 
     Copyright (C) 2004, 2005, 2006, 2007 by Stephen McInerney (spm@stedee.id.au)
     
@@ -33,7 +33,8 @@ const char *css[] = {
     "td {text-align: right; font-size: small; font-weight: bold;}   /* Default setting for ALL table data, everything else is the exception! */",
 
     "td.text {text-align: left; font-weight: normal; white-space: nowrap;}  /* Default text in tables */",
-    "td.percent {font-size: x-small; font-weight: normal;}      /* Percent */",
+    "td.percent {font-size: x-small; font-weight: normal;}          /* Percent */",
+    "td.special_percent {font-size: small; font-weight: normal;}      /* Special Percents */",
 
     "td.front_val {font-weight: normal;}    /* Values on the Main Index Page */",
     "td.front_str {font-weight: normal;}    /* Equiv to Val, used for Volume */",
@@ -82,6 +83,8 @@ const char *css[] = {
     "th.country {background: " CYAN ";}",
     "th.hostname {background: " ORANGE ";}",
     "th.referringurl {background: " LTBLUE ";}",
+    "th.stickiness {background: " BROWN ";}",
+    "th.popularity {background: " LTGREEN ";}",
 
     "img.flags {margin: 0; width: 25px; height: 15px; padding: 0 0.2em 0 0;}",
 
@@ -118,7 +121,7 @@ generate_css_file(void)
     static bool css_exists_warning = false;
     static bool css_created = false;
 
-    cssfile = open(css_filename, O_WRONLY | O_CREAT | O_EXCL, CSS_FILE_PERMISSIONS);
+    cssfile = open(g_settings.settings.css_filename, O_WRONLY | O_CREAT | O_EXCL, CSS_FILE_PERMISSIONS);
     if (cssfile == 0)
         return;
 
@@ -126,7 +129,7 @@ generate_css_file(void)
         snprintf(lineout, sizeof(lineout), "%s\n", css[i]);
         write_rtn = write(cssfile, lineout, strlen(lineout));
         if (write_rtn == -1 && css_exists_warning == false && css_created == false) {
-            ERRVPRINT(VERBOSE1, _("CSS File \"%s\" Exists. Not writing a new one.\n"), css_filename);
+            ERRVPRINT(VERBOSE1, _("CSS File \"%s\" Exists. Not writing a new one.\n"), g_settings.settings.css_filename);
             css_exists_warning = true;
             break;
         }
