@@ -1,10 +1,10 @@
 /***************************************************************************
  *   AWFFull - A Webalizer Fork, Full o' features                          *
  *                                                                         *
- *   Copyright (C) 2004, 2005 by Stephen McInerney                         *
+ *   Copyright (C) 2004, 2005, 2007 by Stephen McInerney                         *
  *   spm@stedee.id.au                                                      *
  *                                                                         *
- *   $Id: graphs.h 392 2007-05-07 11:49:54Z steve $
+ *   $Id: graphs.h 442 2007-08-06 07:28:29Z steve $
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -34,9 +34,8 @@
 #ifndef AWFFULL_GRAPHS_H
 #define AWFFULL_GRAPHS_H
 
-extern int month_graph6(char *, char *, int, int, u_long *, u_long *, u_long *, unsigned long long *, u_long *, u_long *);
+extern int month_graph6(char *, char *, int, int, bool, unsigned long *, unsigned long *, unsigned long *, unsigned long long *, unsigned long *, unsigned long *);
 extern int year_graph6x(char *, char *, int, int);
-extern int day_graph4(char *, char *, u_long *, u_long *, u_long *, unsigned long long *);
 extern int pie_chart(char *, char *, unsigned long long, unsigned long long *, char **);
 extern void initialise_graphs(void);
 
@@ -76,8 +75,8 @@ extern void initialise_graphs(void);
 #define GRAPH_FONT_SIZE_TINY 7.0                /* Size in pixels to use for fonts - Tiny     */
 #define GRAPH_FONT_SIZE_SMALL 8.0               /* Size in pixels to use for fonts - Small     */
 #define GRAPH_FONT_SIZE_MEDIUM 9.0              /* Size in pixels to use for fonts - Medium     */
-#define GRAPH_FONT_DEFAULT "heveltica:bold"     /* Default True Type Font to use        */
-#define GRAPH_FONT_LABEL "heveltica"            /* Default True Type Font to use        */
+#define GRAPH_FONT_DEFAULT "Helvetica:bold"     /* Default True Type Font to use        */
+#define GRAPH_FONT_LABEL "Helvetica"            /* Default True Type Font to use        */
 
 #ifndef FONTDEFAULT
 #define GRAPH_FONT_FULLPATH_DEFAULT "/usr/share/fonts/bitstream-vera/VeraBd.ttf"        /* When all else fails... */
@@ -91,4 +90,5 @@ extern void initialise_graphs(void);
 #define GRAPH_FONT_FULLPATH_LABEL FONTLABEL
 #endif
 
+
 #endif          /* AWFFULL_GRAPHS_H */
