@@ -1,11 +1,11 @@
 /*
     AWFFull - A Webalizer Fork, Full o' features
     
-    $Id: hashtab.c 247 2006-07-15 11:13:59Z steve $
+    $Id: hashtab.c 440 2007-08-05 12:20:41Z steve $
 
     Copyright (C) 1997-2001  Bradford L. Barrett (brad@mrunix.net)
     Copyright (C) 2006 by Alexander Lazic (al-awffull@none.at)
-    Copyright (C) 2006 by Stephen McInerney (spm@stedee.id.au)
+    Copyright (C) 2006, 2007 by Stephen McInerney (spm@stedee.id.au)
     Copyright (C) 2006 by John Heaton (john@manchester.ac.uk)
 
     This program is free software; you can redistribute it and/or modify
@@ -48,9 +48,9 @@ INODEPTR new_inode(char *);                     /* new ident node           */
 ENODEPTR new_enode(char *, char *);             /* new errorpage node       */
 
 void update_entry(char *);                      /* update entry/exit        */
-void update_exit(char *);                       /* page totals              */
+void update_exit(char *, bool);                 /* page totals              */
 
-u_long hash(char *);                            /* hash function            */
+unsigned long hash(char *);                     /* hash function            */
 
 /* local data */
 
@@ -89,142 +89,128 @@ HNODEPTR
 new_hnode(char *str)
 {
     HNODEPTR newptr;
-    char *sptr;
-    size_t len = strlen(str);
+    size_t len = strlen(str) + 1;
 
-    if (len >= MAXHOST) {
+    if (len > MAXHOST) {
         ERRVPRINT(VERBOSE1, "[new_hnode] %s (%d)\n", _("Warning: String exceeds storage size"), (int) len);
         ERRVPRINT(VERBOSE3, "  --> %s\n", str);
-        str[MAXHOST - 1] = 0;
-        len = MAXHOST - 1;
+        str[MAXHOST] = 0;
+        len = MAXHOST;
     }
 
-    if ((sptr = calloc(1, len + 1)) == NULL)
-        return (HNODEPTR) NULL;
-    memcpy(sptr, str, len);
+    newptr = XMALLOC(struct hnode, 1);
+    newptr->string = XMALLOC(char, len);
 
-    if ((newptr = malloc(sizeof(struct hnode))) != NULL) {
-        newptr->string = sptr;
-        newptr->visit = 0;
-        newptr->tstamp = 0;
-        newptr->lasturl = blank_str;
-    } else
-        free(sptr);
+    strlcpy(newptr->string, str, len);
+    newptr->lasturl = blank_str;
     return newptr;
 }
 
-/*********************************************/
-/* PUT_HNODE - insert/update host node       */
-/*********************************************/
+/*********************************************
+ * PUT_HNODE - insert/update host node       
+ * 
+ * if visit > 0; then this is a "load" event from preserve.c
+ * OR is a CreateGroup event from output.c
+ *********************************************/
 
 int
 put_hnode(char *str,                            /* Hostname  */
           int type,                             /* obj type  */
-          u_long count,                         /* hit count */
-          u_long file,                          /* File flag */
+          unsigned long count,                  /* hit count */
+          unsigned long file,                   /* File flag */
           unsigned long long xfer,              /* xfer size */
-          u_long * ctr,                         /* counter   */
-          u_long visit,                         /* visits    */
-          u_long page,                          /* pages         */
-          u_long tstamp,                        /* timestamp */
+          unsigned long *ctr,                   /* counter   */
+          unsigned long visit,                  /* visits    */
+          unsigned long page,                   /* pages     */
+          unsigned long tstamp,                 /* timestamp */
+          char *url,                            /* Current URL */
           char *lasturl,                        /* lasturl */
-          HNODEPTR * htab, bool isapage)
-{                                               /* ptr>next  */
-    HNODEPTR cptr, nptr;
+          HNODEPTR * htab,                      /* Host Hash Table */
+          bool lasturl_is_entry,                /* true if the last URL is an entry page. Only used for creating a new node from preserve/restore */
+          bool isapage,                         /* Is a Page Y/N */
+          bool * isnewsite)
+{
+    HNODEPTR cptr, nptr, prev_cptr;
 
-    /* check if hashed */
-    if ((cptr = htab[hash(str)]) == NULL) {
-        /* not hashed */
-        if ((nptr = new_hnode(str)) != NULL) {
-            nptr->flag = type;
-            nptr->count = count;
-            nptr->files = file;
-            nptr->xfer = xfer;
-            nptr->pages = page;
-            nptr->next = NULL;
-            htab[hash(str)] = nptr;
-            if (type != OBJ_GRP)
-                (*ctr)++;
-
-            if (visit) {
-                nptr->visit = (visit - 1);
-                nptr->lasturl = find_url(lasturl);
-                nptr->tstamp = tstamp;
-                return 0;
-            } else {
-                if (isapage) {
-                    if (htab == sm_htab)
-                        update_entry(log_rec.url);
-                    nptr->lasturl = find_url(log_rec.url);
-                    nptr->tstamp = tstamp;
-                    nptr->visit = 1;
-                }
-            }
-        }
-    } else {
-        /* hashed */
-        while (cptr != NULL) {
-            if (strcmp(cptr->string, str) == 0) {
-                if ((type == cptr->flag) || ((type != OBJ_GRP) && (cptr->flag != OBJ_GRP))) {
-                    /* found... bump counter */
-                    cptr->count += count;
-                    cptr->files += file;
-                    cptr->xfer += xfer;
-                    cptr->pages += page;
-
-                    if (isapage) {
-                        if ((tstamp - cptr->tstamp) >= visit_timeout) {
-                            cptr->visit++;
-                            if (htab == sm_htab) {
-                                update_exit(cptr->lasturl);
-                                update_entry(log_rec.url);
-                            }
+    cptr = htab[hash(str)];
+    prev_cptr = NULL;
+    while (cptr != NULL) {
+        if (strncmp(cptr->string, str, MAXHOST) == 0) {
+            if ((type == cptr->flag) || ((type != OBJ_GRP) && (cptr->flag != OBJ_GRP))) {
+                /* found... bump counters */
+                cptr->count += count;
+                cptr->files += file;
+                cptr->xfer += xfer;
+                cptr->pages += page;
+                if (type == OBJ_GRP) {
+                    /* Updating a group */
+                    cptr->visit += visit;
+                } else if (isapage) {
+                    if ((tstamp - cptr->tstamp) >= g_settings.settings.visit_timeout) {
+                        /* Is a new Visit */
+                        cptr->visit++;
+                        /* ONLY! Update entry/exit if working with Monthly table */
+                        if (htab == sm_htab) {
+                            update_exit(cptr->lasturl, cptr->lasturl_is_entry);
+                            update_entry(url);
+                            cptr->lasturl_is_entry = true;
+                            cptr->lasturl = find_url(url);
+                        }
+                    } else {                    /* Still in same visit */
+                        if (htab == sm_htab) {
+                            cptr->lasturl_is_entry = false;
+                            cptr->lasturl = find_url(url);
                         }
-                        cptr->lasturl = find_url(log_rec.url);
-                        cptr->tstamp = tstamp;
                     }
-                    return 0;
+                    /* Either way, update the last timestamp for this visit - we have a page! */
+                    cptr->tstamp = tstamp;
                 }
+                return 0;
             }
-            cptr = cptr->next;
         }
-        /* not found... */
-        if ((nptr = new_hnode(str)) != NULL) {
-            nptr->flag = type;
-            nptr->count = count;
-            nptr->files = file;
-            nptr->xfer = xfer;
-            nptr->pages = page;
-            nptr->next = htab[hash(str)];
+        prev_cptr = cptr;
+        cptr = cptr->next;
+    }
+    /* Failed to Hash, or doesn't already exist */
+    if ((nptr = new_hnode(str)) != NULL) {
+        nptr->flag = type;
+        nptr->count = count;
+        nptr->files = file;
+        nptr->xfer = xfer;
+        nptr->pages = page;
+        nptr->next = NULL;
+        nptr->lasturl_is_entry = lasturl_is_entry;
+        if (prev_cptr != NULL) {
+            prev_cptr->next = nptr;
+        } else {
             htab[hash(str)] = nptr;
-            if (type != OBJ_GRP)
-                (*ctr)++;
-
-            if (visit) {
-                nptr->visit = (visit - 1);
+        }
+        if (type == OBJ_GRP) {
+            nptr->flag = OBJ_GRP;
+        } else {
+            (*ctr)++;
+            if (!*isnewsite) {
+                *isnewsite = true;
+            }
+        }
+        if (visit) {
+            nptr->visit = visit;
+            if (type != OBJ_GRP) {
                 nptr->lasturl = find_url(lasturl);
                 nptr->tstamp = tstamp;
-                return 0;
-            } else {
-                if (isapage) {
-                    if (htab == sm_htab)
-                        update_entry(log_rec.url);
-                    nptr->lasturl = find_url(log_rec.url);
-                    nptr->tstamp = tstamp;
-                    nptr->visit = 1;
-                }
             }
-        }
-    }
-
-    if (nptr != NULL) {
-        /* set object type */
-        if (type == OBJ_GRP)
-            nptr->flag = OBJ_GRP;               /* is it a grouping? */
-        else {
-            /* check if it's a hidden object */
-            if ((hide_sites) || (isinlist(hidden_sites, nptr->string) != NULL))
-                nptr->flag = OBJ_HIDE;
+            return 0;
+        } else {
+            if (isapage) {
+                /* ONLY! Update entry/exit if working with Monthly table */
+                if (htab == sm_htab) {
+                    update_entry(url);
+                    nptr->lasturl = find_url(url);
+                    nptr->lasturl_is_entry = true;
+                }
+                nptr->tstamp = tstamp;
+                nptr->visit = 1;
+            }
         }
     }
     return nptr == NULL;
@@ -246,8 +232,8 @@ del_hlist(HNODEPTR * htab)
             aptr = htab[i];
             while (aptr != NULL) {
                 temp = aptr->next;
-                free(aptr->string);             /* free hostname string space */
-                free(aptr);                     /* free hostname structure    */
+                XFREE(aptr->string);            /* free hostname string space */
+                XFREE(aptr);                    /* free hostname structure    */
                 aptr = temp;
             }
             htab[i] = NULL;
@@ -263,26 +249,21 @@ UNODEPTR
 new_unode(char *str)
 {
     UNODEPTR newptr;
-    char *sptr;
-    size_t len = strlen(str);
+    size_t len = strlen(str) + 1;
 
-    if (len >= MAXURLH) {
+    if (len > MAXURL) {
         ERRVPRINT(VERBOSE1, "[new_unode] %s (%d)\n", _("Warning: String exceeds storage size"), (int) len);
         ERRVPRINT(VERBOSE3, "  --> %s\n", str);
-        str[MAXURLH - 1] = 0;
-        len = MAXURLH - 1;
+        str[MAXURL] = 0;
+        len = MAXURL;
     }
 
-    if ((sptr = calloc(1, len + 1)) == NULL)
-        return (UNODEPTR) NULL;
-    memcpy(sptr, str, len);
+    newptr = XMALLOC(struct unode, 1);
+    newptr->string = XMALLOC(char, len);
 
-    if ((newptr = malloc(sizeof(struct unode))) != NULL) {
-        newptr->string = sptr;
-        newptr->count = 0;
-        newptr->flag = OBJ_REG;
-    } else
-        free(sptr);
+    strlcpy(newptr->string, str, len);
+    newptr->count = 0;
+    newptr->flag = OBJ_REG;
     return newptr;
 }
 
@@ -291,7 +272,15 @@ new_unode(char *str)
 /*********************************************/
 
 int
-put_unode(char *str, int type, u_long count, unsigned long long xfer, u_long * ctr, u_long entry_page_ctr, u_long exit_page_ctr, UNODEPTR * htab)
+put_unode(char *str,                            /* Hostname  */
+          int type,                             /* obj type  */
+          unsigned long count,                  /* hit count */
+          unsigned long long xfer,              /* xfer size */
+          unsigned long *ctr,                   /* counter   */
+          unsigned long entry_page_ctr,         /* Entry Page Counter */
+          unsigned long exit_page_ctr,          /* Exit Page Counter */
+          unsigned long single_access,          /* Single Access Counter */
+          int resp_code, UNODEPTR * htab)
 {
     UNODEPTR cptr, nptr;
 
@@ -303,11 +292,13 @@ put_unode(char *str, int type, u_long count, unsigned long long xfer, u_long * c
         /* not hashed */
         if ((nptr = new_unode(str)) != NULL) {
             nptr->flag = type;
-            if (log_rec.resp_code == RC_PARTIALCONTENT) {
+            if (resp_code == RC_PARTIALCONTENT) {
                 nptr->count = 0;
                 nptr->xfer = 0;
                 nptr->pcount = count;
                 nptr->pxfer = xfer;
+                g_counters.month.partial_hit += count;
+                g_counters.month.partial_vol += xfer;
             } else {
                 nptr->count = count;
                 nptr->xfer = xfer;
@@ -317,6 +308,7 @@ put_unode(char *str, int type, u_long count, unsigned long long xfer, u_long * c
             nptr->next = NULL;
             nptr->entry = entry_page_ctr;
             nptr->exit = exit_page_ctr;
+            nptr->single_access = single_access;
             htab[hash(str)] = nptr;
             if (type != OBJ_GRP)
                 (*ctr)++;
@@ -327,9 +319,11 @@ put_unode(char *str, int type, u_long count, unsigned long long xfer, u_long * c
             if (strcmp(cptr->string, str) == 0) {
                 if ((type == cptr->flag) || ((type != OBJ_GRP) && (cptr->flag != OBJ_GRP))) {
                     /* found... bump counter */
-                    if (log_rec.resp_code == RC_PARTIALCONTENT) {
+                    if (resp_code == RC_PARTIALCONTENT) {
                         cptr->pcount += count;
                         cptr->pxfer += xfer;
+                        g_counters.month.partial_hit += count;
+                        g_counters.month.partial_vol += xfer;
                     } else {
                         cptr->count += count;
                         cptr->xfer += xfer;
@@ -342,11 +336,13 @@ put_unode(char *str, int type, u_long count, unsigned long long xfer, u_long * c
         /* not found... */
         if ((nptr = new_unode(str)) != NULL) {
             nptr->flag = type;
-            if (log_rec.resp_code == RC_PARTIALCONTENT) {
+            if (resp_code == RC_PARTIALCONTENT) {
                 nptr->count = 0;
                 nptr->xfer = 0;
                 nptr->pcount = count;
                 nptr->pxfer = xfer;
+                g_counters.month.partial_hit += count;
+                g_counters.month.partial_vol += xfer;
             } else {
                 nptr->count = count;
                 nptr->xfer = xfer;
@@ -356,6 +352,7 @@ put_unode(char *str, int type, u_long count, unsigned long long xfer, u_long * c
             nptr->next = htab[hash(str)];
             nptr->entry = entry_page_ctr;
             nptr->exit = exit_page_ctr;
+            nptr->single_access = single_access;
             htab[hash(str)] = nptr;
             if (type != OBJ_GRP)
                 (*ctr)++;
@@ -386,8 +383,8 @@ del_ulist(UNODEPTR * htab)
             aptr = htab[i];
             while (aptr != NULL) {
                 temp = aptr->next;
-                free(aptr->string);             /* free up URL string memory */
-                free(aptr);                     /* free up URL struct node   */
+                XFREE(aptr->string);            /* free up URL string memory */
+                XFREE(aptr);                    /* free up URL struct node   */
                 aptr = temp;
             }
             htab[i] = NULL;
@@ -403,26 +400,21 @@ RNODEPTR
 new_rnode(char *str)
 {
     RNODEPTR newptr;
-    char *sptr;
-    size_t len = strlen(str);
+    size_t len = strlen(str) + 1;
 
-    if (len >= MAXREFH) {
+    if (len >= MAXREF) {
         ERRVPRINT(VERBOSE1, "[new_rnode] %s (%d)\n", _("Warning: String exceeds storage size"), (int) len);
         ERRVPRINT(VERBOSE3, "  --> %s\n", str);
-        str[MAXREFH - 1] = 0;
-        len = MAXREFH - 1;
+        str[MAXREF] = 0;
+        len = MAXREF;
     }
 
-    if ((sptr = calloc(1, len + 1)) == NULL)
-        return (RNODEPTR) NULL;
-    memcpy(sptr, str, len);
+    newptr = XMALLOC(struct rnode, 1);
+    newptr->string = XMALLOC(char, len);
 
-    if ((newptr = malloc(sizeof(struct rnode))) != NULL) {
-        newptr->string = sptr;
-        newptr->count = 1;
-        newptr->flag = OBJ_REG;
-    } else
-        free(sptr);
+    strlcpy(newptr->string, str, len);
+    newptr->count = 1;
+    newptr->flag = OBJ_REG;
     return newptr;
 }
 
@@ -431,7 +423,7 @@ new_rnode(char *str)
 /*********************************************/
 
 int
-put_rnode(char *str, int type, u_long count, u_long * ctr, RNODEPTR * htab)
+put_rnode(char *str, int type, unsigned long count, unsigned long *ctr, RNODEPTR * htab)
 {
     RNODEPTR cptr, nptr;
 
@@ -496,8 +488,8 @@ del_rlist(RNODEPTR * htab)
             aptr = htab[i];
             while (aptr != NULL) {
                 temp = aptr->next;
-                free(aptr->string);
-                free(aptr);
+                XFREE(aptr->string);
+                XFREE(aptr);
                 aptr = temp;
             }
             htab[i] = NULL;
@@ -513,26 +505,21 @@ ANODEPTR
 new_anode(char *str)
 {
     ANODEPTR newptr;
-    char *sptr;
-    size_t len = strlen(str);
+    size_t len = strlen(str) + 1;
 
-    if (len >= MAXAGENT) {
+    if (len > MAXAGENT) {
         ERRVPRINT(VERBOSE1, "[new_anode] %s (%d)\n", _("Warning: String exceeds storage size"), (int) len);
         ERRVPRINT(VERBOSE3, "  --> %s\n", str);
-        str[MAXAGENT - 1] = 0;
-        len = MAXAGENT - 1;
+        str[MAXAGENT] = 0;
+        len = MAXAGENT;
     }
 
-    if ((sptr = calloc(1, len + 1)) == NULL)
-        return (ANODEPTR) NULL;
-    memcpy(sptr, str, len);
+    newptr = XMALLOC(struct anode, 1);
+    newptr->string = XMALLOC(char, len);
 
-    if ((newptr = malloc(sizeof(struct anode))) != NULL) {
-        newptr->string = sptr;
-        newptr->count = 1;
-        newptr->flag = OBJ_REG;
-    } else
-        free(sptr);
+    strlcpy(newptr->string, str, len);
+    newptr->count = 1;
+    newptr->flag = OBJ_REG;
     return newptr;
 }
 
@@ -541,7 +528,7 @@ new_anode(char *str)
 /*********************************************/
 
 int
-put_anode(char *str, int type, u_long count, u_long * ctr, ANODEPTR * htab)
+put_anode(char *str, int type, unsigned long count, unsigned long *ctr, ANODEPTR * htab)
 {
     ANODEPTR cptr, nptr;
 
@@ -604,8 +591,8 @@ del_alist(ANODEPTR * htab)
             aptr = htab[i];
             while (aptr != NULL) {
                 temp = aptr->next;
-                free(aptr->string);
-                free(aptr);
+                XFREE(aptr->string);
+                XFREE(aptr);
                 aptr = temp;
             }
             htab[i] = NULL;
@@ -621,25 +608,20 @@ SNODEPTR
 new_snode(char *str)
 {
     SNODEPTR newptr;
-    char *sptr;
-    size_t len = strlen(str);
+    size_t len = strlen(str) + 1;
 
-    if (len >= MAXSRCHH) {
+    if (len > MAXSRCHH) {
         ERRVPRINT(VERBOSE1, "[new_snode] %s (%d)\n", _("Warning: String exceeds storage size"), (int) len);
         ERRVPRINT(VERBOSE3, "  --> %s\n", str);
-        str[MAXSRCHH - 1] = 0;
-        len = MAXSRCHH - 1;
+        str[MAXSRCHH] = 0;
+        len = MAXSRCHH;
     }
 
-    if ((sptr = calloc(1, len + 1)) == NULL)
-        return (SNODEPTR) NULL;
-    memcpy(sptr, str, len);
+    newptr = XMALLOC(struct snode, 1);
+    newptr->string = XMALLOC(char, len);
 
-    if ((newptr = malloc(sizeof(struct snode))) != NULL) {
-        newptr->string = sptr;
-        newptr->count = 1;
-    } else
-        free(sptr);
+    strlcpy(newptr->string, str, len);
+    newptr->count = 1;
     return newptr;
 }
 
@@ -648,7 +630,7 @@ new_snode(char *str)
 /*********************************************/
 
 int
-put_snode(char *str, u_long count, SNODEPTR * htab)
+put_snode(char *str, unsigned long count, SNODEPTR * htab)
 {
     SNODEPTR cptr, nptr;
 
@@ -699,8 +681,8 @@ del_slist(SNODEPTR * htab)
             aptr = htab[i];
             while (aptr != NULL) {
                 temp = aptr->next;
-                free(aptr->string);
-                free(aptr);
+                XFREE(aptr->string);
+                XFREE(aptr);
                 aptr = temp;
             }
             htab[i] = NULL;
@@ -716,26 +698,21 @@ INODEPTR
 new_inode(char *str)
 {
     INODEPTR newptr;
-    char *sptr;
-    size_t len = strlen(str);
+    size_t len = strlen(str) + 1;
 
-    if (len >= MAXIDENT) {
+    if (len > MAXIDENT) {
         ERRVPRINT(VERBOSE1, "[new_inode] %s (%d)\n", _("Warning: String exceeds storage size"), (int) len);
         ERRVPRINT(VERBOSE1, "  --> %s\n", str);
-        str[MAXIDENT - 1] = 0;
-        len = MAXIDENT - 1;
+        str[MAXIDENT] = 0;
+        len = MAXIDENT;
     }
 
-    if ((sptr = calloc(1, len + 1)) == NULL)
-        return (INODEPTR) NULL;
-    memcpy(sptr, str, len);
+    newptr = XMALLOC(struct inode, 1);
+    newptr->string = XMALLOC(char, len);
 
-    if ((newptr = malloc(sizeof(struct inode))) != NULL) {
-        newptr->string = sptr;
-        newptr->visit = 1;
-        newptr->tstamp = 0;
-    } else
-        free(sptr);
+    strlcpy(newptr->string, str, len);
+    newptr->visit = 1;
+    newptr->tstamp = 0;
     return newptr;
 }
 
@@ -746,12 +723,12 @@ new_inode(char *str)
 int
 put_inode(char *str,                            /* ident str */
           int type,                             /* obj type  */
-          u_long count,                         /* hit count */
-          u_long file,                          /* File flag */
+          unsigned long count,                  /* hit count */
+          unsigned long file,                   /* File flag */
           unsigned long long xfer,              /* xfer size */
-          u_long * ctr,                         /* counter   */
-          u_long visit,                         /* visits    */
-          u_long tstamp,                        /* timestamp */
+          unsigned long *ctr,                   /* counter   */
+          unsigned long visit,                  /* visits    */
+          unsigned long tstamp,                 /* timestamp */
           INODEPTR * htab, bool isapage)
 {                                               /* hashtable */
     INODEPTR cptr, nptr;
@@ -773,7 +750,7 @@ put_inode(char *str,                            /* ident str */
                 (*ctr)++;
 
             if (visit) {
-                nptr->visit = (visit - 1);
+                nptr->visit = visit;
                 nptr->tstamp = tstamp;
                 return 0;
             } else {
@@ -792,9 +769,8 @@ put_inode(char *str,                            /* ident str */
                     cptr->files += file;
                     cptr->xfer += xfer;
 
-//                    if (parse_is_page (log_rec.url)) {
                     if (isapage) {
-                        if ((tstamp - cptr->tstamp) >= visit_timeout)
+                        if ((tstamp - cptr->tstamp) >= g_settings.settings.visit_timeout)
                             cptr->visit++;
                         cptr->tstamp = tstamp;
                     }
@@ -815,7 +791,7 @@ put_inode(char *str,                            /* ident str */
                 (*ctr)++;
 
             if (visit) {
-                nptr->visit = (visit - 1);
+                nptr->visit = visit;
                 nptr->tstamp = tstamp;
                 return 0;
             } else {
@@ -855,8 +831,8 @@ del_ilist(INODEPTR * htab)
             aptr = htab[i];
             while (aptr != NULL) {
                 temp = aptr->next;
-                free(aptr->string);             /* free ident string space */
-                free(aptr);                     /* free ident structure    */
+                XFREE(aptr->string);            /* free ident string space */
+                XFREE(aptr);                    /* free ident structure    */
                 aptr = temp;
             }
             htab[i] = NULL;
@@ -884,8 +860,8 @@ new_enode(char *url,                            /* url str   */
         len_url = MAXURL - 1;
     }
 
-    if ((url_ptr = calloc(1, len_url + 1)) == NULL)
-        return (ENODEPTR) NULL;
+    url_ptr = XMALLOC(char, len_url + 1);
+
     memcpy(url_ptr, url, len_url);
 
     if (len_ref >= MAXURL) {
@@ -895,27 +871,23 @@ new_enode(char *url,                            /* url str   */
         len_ref = MAXURL - 1;
     }
 
-    if ((ref_ptr = calloc(1, len_ref + 1)) == NULL)
-        return (ENODEPTR) NULL;
+    ref_ptr = XMALLOC(char, len_ref + 1);
+
     memcpy(ref_ptr, refer, len_ref);
 
     /* Hash key can be sized MAXURL x 2 */
-    if ((key_ptr = calloc(1, len_url + len_ref + 1)) == NULL)
-        return (ENODEPTR) NULL;
+    key_ptr = XMALLOC(char, len_url + len_ref + 1);
+
     memcpy(key_ptr, url, len_url);
     strncat(key_ptr, refer, len_ref);
 
-    if ((newptr = malloc(sizeof(struct enode))) != NULL) {
-        newptr->error_request = key_ptr;
-        newptr->url = url_ptr;
-        newptr->referer = ref_ptr;
-        newptr->count = 1;
-        newptr->next = NULL;
-    } else {
-        free(url_ptr);
-        free(ref_ptr);
-        free(key_ptr);
-    }
+    newptr = XMALLOC(struct enode, 1);
+
+    newptr->error_request = key_ptr;
+    newptr->url = url_ptr;
+    newptr->referer = ref_ptr;
+    newptr->count = 1;
+    newptr->next = NULL;
     return newptr;
 }
 
@@ -927,8 +899,8 @@ int
 put_enode(char *url,                            /* url str   */
           char *refer,                          /* my referer */
           int type,                             /* obj type  */
-          u_long count,                         /* hit count */
-          u_long * ctr,                         /* counter   */
+          unsigned long count,                  /* hit count */
+          unsigned long *ctr,                   /* counter   */
           ENODEPTR * htab)
 {                                               /* hashtable */
     ENODEPTR cptr, nptr;
@@ -937,8 +909,8 @@ put_enode(char *url,                            /* url str   */
     size_t len_ref = strlen(refer);
     unsigned long hash_val;
 
-    if ((key_ptr = calloc(1, len_url + len_ref + 1)) == NULL)
-        return (0);
+    key_ptr = XMALLOC(char, len_url + len_ref + 1);
+
     memcpy(key_ptr, url, len_url);
     strncat(key_ptr, refer, len_ref);
 
@@ -951,7 +923,7 @@ put_enode(char *url,                            /* url str   */
             htab[hash_val] = nptr;
             (*ctr)++;
             nptr->count = count;
-            free(key_ptr);
+            XFREE(key_ptr);
             return 0;
         }
     } else {
@@ -960,7 +932,7 @@ put_enode(char *url,                            /* url str   */
             if (strcmp(cptr->error_request, key_ptr) == 0) {
                 /* found... bump counter */
                 cptr->count += count;
-                free(key_ptr);
+                XFREE(key_ptr);
                 return 0;
             }
             cptr = cptr->next;
@@ -973,7 +945,7 @@ put_enode(char *url,                            /* url str   */
         }
     }
 
-    free(key_ptr);
+    XFREE(key_ptr);
     return nptr == NULL;
 }
 
@@ -994,18 +966,18 @@ del_elist(ENODEPTR * htab)
             while (aptr != NULL) {
                 temp = aptr->next;
                 if (aptr->error_request) {
-                    free(aptr->error_request);  /* free errorpage string space */
+                    XFREE(aptr->error_request); /* free errorpage string space */
                     aptr->error_request = NULL;
                 }
                 if (aptr->referer) {
-                    free(aptr->referer);        /* free referer string space */
+                    XFREE(aptr->referer);       /* free referer string space */
                     aptr->referer = NULL;
                 }
                 if (aptr->url) {
-                    free(aptr->url);            /* free referer string space */
+                    XFREE(aptr->url);           /* free referer string space */
                     aptr->url = NULL;
                 }
-                free(aptr);                     /* free errorpage structure    */
+                XFREE(aptr);                    /* free errorpage structure    */
                 aptr = temp;
             }
             htab[i] = NULL;
@@ -1017,10 +989,10 @@ del_elist(ENODEPTR * htab)
 /* HASH - return hash value for string       */
 /*********************************************/
 
-u_long
+unsigned long
 hash(char *str)
 {
-    u_long hashval = MAXHASH;
+    unsigned long hashval = MAXHASH;
 
     while (*str != '\0') {
         hashval = (hashval << 5) + hashval + *str;
@@ -1079,7 +1051,7 @@ update_entry(char *str)
 /*********************************************/
 
 void
-update_exit(char *str)
+update_exit(char *str, bool is_single_access)
 {
     UNODEPTR uptr;
 
@@ -1092,6 +1064,9 @@ update_exit(char *str)
             if (strcmp(uptr->string, str) == 0) {
                 if (uptr->flag != OBJ_GRP) {
                     uptr->exit++;
+                    if (is_single_access) {
+                        uptr->single_access++;
+                    }
                     return;
                 }
             }
@@ -1105,7 +1080,7 @@ update_exit(char *str)
 /*********************************************/
 
 void
-month_update_exit(u_long tstamp)
+month_update_exit(unsigned long tstamp)
 {
     HNODEPTR nptr;
     int i;
@@ -1114,8 +1089,10 @@ month_update_exit(u_long tstamp)
         nptr = sm_htab[i];
         while (nptr != NULL) {
             if (nptr->flag != OBJ_GRP) {
-                if ((tstamp - nptr->tstamp) >= visit_timeout)
-                    update_exit(nptr->lasturl);
+                if ((tstamp - nptr->tstamp) >= g_settings.settings.visit_timeout) {
+                    /* End of month, update all "still open" sessions; and count as closed sessions that would be expired */
+                    update_exit(nptr->lasturl, nptr->lasturl_is_entry);
+                }
             }
             nptr = nptr->next;
         }
@@ -1126,18 +1103,19 @@ month_update_exit(u_long tstamp)
 /* TOT_VISIT - calculate total visits        */
 /*********************************************/
 
-u_long
+unsigned long
 tot_visit(HNODEPTR * list)
 {
     HNODEPTR hptr;
-    u_long tot = 0;
+    unsigned long tot = 0;
     int i;
 
     for (i = 0; i < MAXHASH; i++) {
         hptr = list[i];
         while (hptr != NULL) {
-            if (hptr->flag != OBJ_GRP)
+            if (hptr->flag != OBJ_GRP) {
                 tot += hptr->visit;
+            }
             hptr = hptr->next;
         }
     }
