@@ -1,26 +1,38 @@
 /*
-   awffull_lang.c
-   
-   $Id: messages.c 269 2006-09-07 10:59:35Z steve $
+    AWFFull - A Webalizer Fork, Full o' features
+    
+    Copyright (C) 2004, 2005, 2006, 2007 by Stephen McInerney (spm@stedee.id.au)
 
-   Webalizer V2.0x Language Support file for English.
-   15-May-1998 by Bradford L. Barrett (brad@mrunix.net)
-   31-May-1998 Modified for level 1.1 support (brad@mrunix.net)
-   23-Jul-1998 Modified for level 1.2 support (brad@mrunix.net)
-   08-Mar-1999 Updated HTTP 1.1 response codes by Yves Lafon (ylafon@w3.org)
-   28-Jun-1999 Modified for level 1.3 support (brad@mrunix.net)
-   16-Feb-2000 Modified for level 2.0 support (brad@mrunix.net)
+    $Id: messages.c 469 2007-11-07 09:54:13Z steve $
 
-   28-Jun-2006 Start conversion to gettext, by Steve McInerney
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version, and provided that the above
+    copyright and permission notice is included with all distributed
+    copies of this or derived software.
 
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
+
+/***************************************************************************
+ ***************************************************************************
+ * messages.c
+ *
+ * More complex messages vs in line translations et al.
+ * Formally 'awffull_lang.c'
+ ***************************************************************************
+ ***************************************************************************/
+
 #include "awffull.h"                            /* main header              */
 
-/***********************************************************************/
-/* DEFINE LANGUAGE NAME here                                           */
-/***********************************************************************/
-
-const char *msg_title;
 
 const char *h_msg;
 const char *s_month[12];
@@ -30,10 +42,6 @@ struct country_code ctry[300];
 struct response_code response[TOTAL_RC];
 
 
-/***********************************************************************/
-/***********************************************************************/
-/***********************************************************************/
-/***********************************************************************/
 /***********************************************************************/
 
 void
@@ -42,8 +50,6 @@ assign_messages(void)
     unsigned int i = 0;                         /* Basic counters for use with INIT_CTRY macro */
     unsigned int j = 0;
 
-    msg_title = _("Usage Statistics for");
-
     h_msg = _("\
   -h, --help                 Print this help message\n\
   -d, --debug                Print additional debug info\n\
@@ -62,14 +68,12 @@ assign_messages(void)
   -o, --output=DIR           Output directory to use\n\
   -T, --timing               Print timing information\n\
 \n\
-  -q, --quiet                supress informational messages\n\
-  -Q, --vquiet               supress _ALL_ messages\n\
-  -Y                         supress country graph\n\
-  -G                         supress hourly graph\n\
-  -H                         supress hourly stats\n\
-  -L                         supress color coded graph legends\n\
+  -Y                         Suppress country graph\n\
+  -G                         Suppress hourly graph\n\
+  -H                         Suppress hourly stats\n\
+  -L                         Suppress color coded graph legends\n\
+  -l                         Suppress background lines on graph\n\
 \n\
-  -l num                     use num background lines on graph\n\
   -n name                    hostname to use\n\
   -t name                    report title 'name'\n\
   -a name                    hide user agent 'name'\n\
@@ -80,13 +84,13 @@ assign_messages(void)
   -P name                    Page type extension 'name'\n\
   -I name                    Index alias 'name'\n\
 \n\
-  -A num                     Display num top agents\n\
-  -C num                     Display num top countries\n\
-  -R num                     Display num top referrers\n\
-  -S num                     Display num top sites\n\
-  -U num                     Display num top URLs\n\
-  -e num                     Display num top Entry Pages\n\
-  -E num                     Display num top Exit Pages\n\
+  -A --top_agents=num        Display num top agents\n\
+  -C --top_countries=num     Display num top countries\n\
+  -R --top_refers=num        Display num top referrers\n\
+  -S --top_sites=num         Display num top sites\n\
+  -U --top_urls=num          Display num top URLs\n\
+  -e --top_entry=num         Display num top Entry Pages\n\
+  -E --top_exit=num          Display num top Exit Pages\n\
   -g num                     Group Domains to 'num' levels\n\
 \n\
   -X                         Hide individual sites\n\
