@@ -1,4 +1,4 @@
-/* $Id: output.h 207 2006-06-04 08:27:18Z steve $ */
+/* $Id: output.h 309 2006-10-22 11:38:07Z steve $ */
 #ifndef AWFFULL_OUTPUT_H
 #define AWFFULL_OUTPUT_H
 
@@ -21,6 +21,8 @@ extern FILE *open_out_file(char *);             /* open output file    */
 #define PURPLE         "#FF00FF"
 #define CYAN           "#00E0FF"
 #define GRPCOLOR       "#D0D0E0"
+#define BROWN          "#FFC480"
+#define LTGREEN        "#80FFC0"
 
 #define MAX_404_RECORDS 1000
 
