@@ -1,7 +1,7 @@
 /*
     AWFFull - A Webalizer Fork, Full o' features
     
-    $Id: parser.c 276 2006-09-11 11:51:07Z steve $
+    $Id: parser.c 440 2007-08-05 12:20:41Z steve $
 
     Copyright (C) 1997-2001  Bradford L. Barrett (brad@mrunix.net)
     Copyright (C) 2004, 2005, 2006 by Stephen McInerney (spm@stedee.id.au)
@@ -42,25 +42,23 @@
 #include "awffull.h"                            /* main header              */
 
 /* internal function prototypes */
-void fmt_logrec(char *);
-int parse_record_web(char *);
-int parse_record_ftp(char *);
-int parse_record_squid(char *);
+static int parse_record_web(char *, struct log_struct *);
+static int parse_record_ftp(char *, struct log_struct *);
+static int parse_record_squid(char *, struct log_struct *);
 
-int identify_log_format(char *);
+static int identify_log_format(char *);
+static void re_compile_all_regexes(void);       /* Use at first run - compiles all used regex's */
+static void re_compile_failed(int, const char *, char *);       /* Display a failed RE Compile & where */
+static void re_check_errors(int);               /* After an RE check, deal with any errors */
+static void error_substring_extract(int, int);  /* Error when we fail on getting a substring */
 
-void re_compile_all_regexes(void);              /* Use at first run - compiles all used regex's */
-void re_compile_failed(int, const char *, char *);      /* Display a failed RE Compile & where */
-void re_check_errors(int);                      /* After an RE check, deal with any errors */
-void error_substring_extract(int, int);         /* Error when we fail on getting a substring */
-
-pcre *cmp_log_regexp = NULL;                    /* Main compiled RE - use as pointer only to one of the below */
-pcre *cmp_log_regexp_clf = NULL;                /* CLF compiled RE */
-pcre *cmp_log_regexp_combined = NULL;           /* Combined compiled RE */
-pcre *cmp_log_regexp_combined_enhanced = NULL;  /* Enhanced Combined compiled RE */
-pcre *cmp_log_regexp_xferlog = NULL;            /* FTP, xferlog format compiled RE */
-pcre *cmp_log_regexp_squid = NULL;              /* SQUID format compiled RE */
-pcre *cmp_log_regexp_domino = NULL;             /* Lotus Domino v6 format compiled RE */
+static pcre *cmp_log_regexp = NULL;             /* Main compiled RE - use as pointer only to one of the below */
+static pcre *cmp_log_regexp_clf = NULL;         /* CLF compiled RE */
+static pcre *cmp_log_regexp_combined = NULL;    /* Combined compiled RE */
+static pcre *cmp_log_regexp_combined_enhanced = NULL;   /* Enhanced Combined compiled RE */
+static pcre *cmp_log_regexp_xferlog = NULL;     /* FTP, xferlog format compiled RE */
+static pcre *cmp_log_regexp_squid = NULL;       /* SQUID format compiled RE */
+static pcre *cmp_log_regexp_domino = NULL;      /* Lotus Domino v6 format compiled RE */
 
 //pcre_extra *studied_log_regexp = NULL;
 
@@ -70,25 +68,26 @@ pcre *cmp_log_regexp_domino = NULL;             /* Lotus Domino v6 format compil
 /*********************************************/
 
 int
-parse_record(char *buffer)
+parse_record(char *buffer, struct log_struct *log_ptr)
+//parse_record(char *buffer)
 {
     int auto_log_type = 0;
 
     /* clear out structure */
-    memset(&log_rec, 0, sizeof(struct log_struct));
+//    memset(&log_rec, 0, sizeof(struct log_struct));
 
     if (cmp_log_regexp == NULL) {
         re_compile_all_regexes();
-        if (log_type == LOG_AUTO) {
+        if (g_settings.settings.log_type == LOG_AUTO) {
             auto_log_type = identify_log_format(buffer);
             if (auto_log_type > 0) {
-                log_type = auto_log_type;
+                g_settings.settings.log_type = auto_log_type;
             } else {
                 ERRVPRINT(VERBOSE0, "%s\n", _("Cannot recognise log format. Manually configure \"LogType\" in the config file."));
                 exit(1);
             }
         }
-        switch (log_type) {
+        switch (g_settings.settings.log_type) {
         case LOG_FTP:
             cmp_log_regexp = cmp_log_regexp_xferlog;
             break;
@@ -105,23 +104,23 @@ parse_record(char *buffer)
             cmp_log_regexp = cmp_log_regexp_domino;
             break;
         default:
-            ERRVPRINT(VERBOSE0, "%s %d\n", _("Unknown LOG Type Setting.:"), log_type);
+            ERRVPRINT(VERBOSE0, "%s %d\n", _("Unknown LOG Type Setting.:"), g_settings.settings.log_type);
             exit(1);
         }
     }
 
     /* call appropriate handler */
-    switch (log_type) {
+    switch (g_settings.settings.log_type) {
     default:
     case LOG_CLF:
     case LOG_COMBINED:
-        return parse_record_web(buffer);
+        return parse_record_web(buffer, log_ptr);
         break;                                  /* clf   */
     case LOG_FTP:
-        return parse_record_ftp(buffer);
+        return parse_record_ftp(buffer, log_ptr);
         break;                                  /* ftp   */
     case LOG_SQUID:
-        return parse_record_squid(buffer);
+        return parse_record_squid(buffer, log_ptr);
         break;                                  /* squid */
     }
 }
@@ -129,8 +128,8 @@ parse_record(char *buffer)
 /*********************************************/
 /* PARSE_RECORD_FTP - ftp log handler        */
 /*********************************************/
-int
-parse_record_ftp(char *buffer)
+static int
+parse_record_ftp(char *buffer, struct log_struct *log_rec)
 {
     int ovector[OVECCOUNT];                     /* RE substring offsets array */
     int rc;                                     /* RE Check return value */
@@ -138,8 +137,8 @@ parse_record_ftp(char *buffer)
 
     int buffer_length;
 
-    char tmp_bytes[25];
-    char completion_status[2];
+    char tmp_bytes[25 + 1];
+    char completion_status[2 + 1];
 
     buffer_length = (int) strlen(buffer);
     rc = pcre_exec(cmp_log_regexp, NULL, buffer, buffer_length, 0, 0, ovector, OVECCOUNT);
@@ -149,7 +148,7 @@ parse_record_ftp(char *buffer)
         return (0);
     }
 
-    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 1, log_rec.datetime, 29);
+    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 1, log_rec->datetime, 29);
     if (copy_substr_rtn < 0) {
         error_substring_extract(copy_substr_rtn, 1);
         return (0);
@@ -157,7 +156,7 @@ parse_record_ftp(char *buffer)
 
     /* Ignore time taken (in seconds) for now... */
 
-    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 3, log_rec.hostname, MAXHOST);
+    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 3, log_rec->hostname, MAXHOST);
     if (copy_substr_rtn < 0) {
         error_substring_extract(copy_substr_rtn, 3);
         return (0);
@@ -168,10 +167,10 @@ parse_record_ftp(char *buffer)
         error_substring_extract(copy_substr_rtn, 4);
         return (0);
     }
-    log_rec.xfer_size = strtoul(tmp_bytes, NULL, 10);
+    log_rec->xfer_size = strtoul(tmp_bytes, NULL, 10);
 
     /* URL */
-    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 5, log_rec.url, MAXURL);
+    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 5, log_rec->url, MAXURL);
     if (copy_substr_rtn < 0) {
         error_substring_extract(copy_substr_rtn, 5);
         return (0);
@@ -182,7 +181,7 @@ parse_record_ftp(char *buffer)
     /* Ignore Direction */
 
     /* User */
-    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 10, log_rec.ident, MAXIDENT);
+    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 10, log_rec->ident, MAXIDENT);
     if (copy_substr_rtn < 0) {
         error_substring_extract(copy_substr_rtn, 10);
         return (0);
@@ -195,10 +194,10 @@ parse_record_ftp(char *buffer)
         return (0);
     }
     if (completion_status[0] == 'i') {
-        log_rec.resp_code = 206;
+        log_rec->resp_code = 206;
     } else {
         /* == c */
-        log_rec.resp_code = 200;
+        log_rec->resp_code = 200;
     }
 
     return (1);
@@ -208,8 +207,8 @@ parse_record_ftp(char *buffer)
  * PARSE_RECORD_WEB - web log handler        *
  * parse with pcre							 *
  *********************************************/
-int
-parse_record_web(char *buffer)
+static int
+parse_record_web(char *buffer, struct log_struct *log_rec)
 {
     int ovector[OVECCOUNT];                     /* RE substring offsets array */
     int rc;                                     /* RE Check return value */
@@ -217,8 +216,8 @@ parse_record_web(char *buffer)
 
     int buffer_length;
 
-    char tmp_status[5];
-    char tmp_bytes[20];
+    char tmp_status[5 + 1];
+    char tmp_bytes[20 + 1];
 
     buffer_length = (int) strlen(buffer);
 //    rc = pcre_exec (cmp_log_regexp, studied_log_regexp, buffer, buffer_length, 0, 0, ovector, OVECCOUNT);
@@ -230,13 +229,13 @@ parse_record_web(char *buffer)
          *   If this works - switch to Domino Logs,
          *   If Fails - Boom.
          */
-        if ((cmp_log_regexp != cmp_log_regexp_domino) && (cmp_log_regexp == cmp_log_regexp_combined) && (log_type == LOG_COMBINED)) {
+        if ((cmp_log_regexp != cmp_log_regexp_domino) && (cmp_log_regexp == cmp_log_regexp_combined) && (g_settings.settings.log_type == LOG_COMBINED)) {
             /* Attempt an enhanced log match */
             VPRINT(VERBOSE1, "%s\n", _("Attempting COMBINED_ENHANCED Regular Expression"));
             rc = pcre_exec(cmp_log_regexp_combined_enhanced, NULL, buffer, buffer_length, 0, 0, ovector, OVECCOUNT);
 
             /* Didn't work. Try Domino? */
-            if (rc < 0 && force_log_type == false) {
+            if (rc < 0 && g_settings.flags.force_log_type == false) {
                 /* Try a domino log format first - if is, switch to using domino checks instead */
                 VPRINT(VERBOSE1, "%s\n", _("Attempting COMBINED_DOMINO Regular Expression"));
                 rc = pcre_exec(cmp_log_regexp_domino, NULL, buffer, buffer_length, 0, 0, ovector, OVECCOUNT);
@@ -254,25 +253,25 @@ parse_record_web(char *buffer)
         }
     }
 
-    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_ADDRESS, log_rec.hostname, MAXHOST);
+    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_ADDRESS, log_rec->hostname, MAXHOST - 1);
     if (copy_substr_rtn < 0) {
         error_substring_extract(copy_substr_rtn, LF_NCSA_ADDRESS);
         return (0);
     }
 
-    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_AUTHUSER, log_rec.ident, MAXIDENT);
+    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_AUTHUSER, log_rec->ident, MAXIDENT - 1);
     if (copy_substr_rtn < 0) {
         error_substring_extract(copy_substr_rtn, LF_NCSA_USER);
         return (0);
     }
 
-    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_DATE_TIME, log_rec.datetime, SIZE_DATE_TIME);
+    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_DATE_TIME, log_rec->datetime, MAXDATETIME - 1);
     if (copy_substr_rtn < 0) {
         error_substring_extract(copy_substr_rtn, LF_NCSA_DATE_TIME);
         return (0);
     }
 
-    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_URL, log_rec.url, MAXURL);
+    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_URL, log_rec->url, MAXURL - 1);
     if (copy_substr_rtn < 0) {
         error_substring_extract(copy_substr_rtn, LF_NCSA_URL);
         return (0);
@@ -283,22 +282,22 @@ parse_record_web(char *buffer)
         error_substring_extract(copy_substr_rtn, LF_NCSA_STATUS);
         return (0);
     }
-    log_rec.resp_code = atoi(tmp_status);
+    log_rec->resp_code = atoi(tmp_status);
 
     copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_BYTES, tmp_bytes, 20);
     if (copy_substr_rtn < 0) {
         error_substring_extract(copy_substr_rtn, LF_NCSA_BYTES);
         return (0);
     }
-    log_rec.xfer_size = strtoul(tmp_bytes, NULL, 10);
+    log_rec->xfer_size = strtoul(tmp_bytes, NULL, 10);
 
-    if (log_type == LOG_COMBINED) {
-        copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_REFERER, log_rec.refer, MAXREF);
+    if (g_settings.settings.log_type == LOG_COMBINED) {
+        copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_REFERER, log_rec->refer, MAXREF - 1);
         if (copy_substr_rtn < 0) {
             error_substring_extract(copy_substr_rtn, LF_NCSA_REFERER);
             return (0);
         }
-        copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_BROWSER, log_rec.agent, MAXAGENT);
+        copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, LF_NCSA_BROWSER, log_rec->agent, MAXAGENT - 1);
         if (copy_substr_rtn < 0) {
             error_substring_extract(copy_substr_rtn, LF_NCSA_BROWSER);
             return (0);
@@ -311,8 +310,8 @@ parse_record_web(char *buffer)
 /*********************************************/
 /* PARSE_RECORD_SQUID - squid log handler    */
 /*********************************************/
-int
-parse_record_squid(char *buffer)
+static int
+parse_record_squid(char *buffer, struct log_struct *log_rec)
 {
     int ovector[OVECCOUNT];                     /* RE substring offsets array */
     int rc;                                     /* RE Check return value */
@@ -320,7 +319,7 @@ parse_record_squid(char *buffer)
 
     int buffer_length;
 
-    char tmp_bytes[25];
+    char tmp_bytes[25 + 1];
 
     buffer_length = (int) strlen(buffer);
     rc = pcre_exec(cmp_log_regexp, NULL, buffer, buffer_length, 0, 0, ovector, OVECCOUNT);
@@ -330,13 +329,13 @@ parse_record_squid(char *buffer)
         return (0);
     }
 
-    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 1, log_rec.datetime, 29);
+    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 1, log_rec->datetime, 29);
     if (copy_substr_rtn < 0) {
         error_substring_extract(copy_substr_rtn, 1);
         return (0);
     }
 
-    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 4, log_rec.hostname, MAXHOST);
+    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 4, log_rec->hostname, MAXHOST - 1);
     if (copy_substr_rtn < 0) {
         error_substring_extract(copy_substr_rtn, 1);
         return (0);
@@ -347,16 +346,16 @@ parse_record_squid(char *buffer)
         error_substring_extract(copy_substr_rtn, 1);
         return (0);
     }
-    log_rec.resp_code = strtoul(tmp_bytes, NULL, 10);
+    log_rec->resp_code = strtoul(tmp_bytes, NULL, 10);
 
     copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 7, tmp_bytes, 20);
     if (copy_substr_rtn < 0) {
         error_substring_extract(copy_substr_rtn, 1);
         return (0);
     }
-    log_rec.xfer_size = strtoul(tmp_bytes, NULL, 10);
+    log_rec->xfer_size = strtoul(tmp_bytes, NULL, 10);
 
-    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 9, log_rec.url, MAXURL);
+    copy_substr_rtn = pcre_copy_substring(buffer, ovector, rc, 9, log_rec->url, MAXURL - 1);
     if (copy_substr_rtn < 0) {
         error_substring_extract(copy_substr_rtn, 1);
         return (0);
@@ -379,7 +378,7 @@ parse_record_squid(char *buffer)
  *                                                                      *
  * Arguments:                                                           *
  * -----------                                                          *
- * char *url      -  A URL field, typically: log_rec.url                *
+ * char *url      -  A URL field, typically: log_rec->url                *
  *                                                                      *
  * Returns:                                                             *
  * -----------                                                          *
@@ -390,7 +389,7 @@ parse_record_squid(char *buffer)
 bool
 parse_check_not_page(char *url)
 {
-    char regex_page[MAX_RE_LENGTH] = "";        /* Hold the PAGE RE */
+    char regex_page[MAX_RE_LENGTH + 1] = "";    /* Hold the PAGE RE */
     static pcre *cmp_regex_page = NULL;         /* NotPage compiled RE */
 
     const char *error;                          /* RE error pointer, offset */
@@ -462,7 +461,7 @@ parse_check_not_page(char *url)
  *                                                                      *
  * Arguments:                                                           *
  * -----------                                                          *
- * char *url      -  A URL field, typically: log_rec.url                *
+ * char *url      -  A URL field, typically: log_rec->url                *
  *                                                                      *
  * Returns:                                                             *
  * -----------                                                          *
@@ -474,14 +473,14 @@ parse_check_not_page(char *url)
 bool
 parse_is_page(char *url)
 {
-    char regex_page[MAX_RE_LENGTH] = "";        /* Hold the PAGE RE */
+    char regex_page[MAX_RE_LENGTH + 1] = "";    /* Hold the PAGE RE */
     static pcre *cmp_regex_page = NULL;         /* Page compiled RE */
 
     const char *error;                          /* RE error pointer, offset */
     int erroffset;                              /* RE error value */
     int str_length;
     int rc;                                     /* RE Check return value */
-    char reverse[MAXURL] = "";
+    char reverse[MAXURL + 1] = "";
     int i, j = 0;
     LISTPTR lptr;
 
@@ -543,88 +542,6 @@ parse_is_page(char *url)
 }                                               /* parse_is_page */
 
 
-/************************************************************************
- * parse_is_bookmark                                                    *
- *                                                                      *
- * Given a URL and a BrowserID (respective fields from a log line)      *
- *   determine if these entries indicate a Bookmark being made          *
- *                                                                      *
- * Relies on the action taken by MSIE to grab a favicon.ico on          *
- *   bookmarking                                                        *
- *                                                                      *
- * Arguments:                                                           *
- * -----------                                                          *
- * char *url      -  A URL field, typically: log_rec.url                *
- * char *browser  -  A Browser ID field, typically: log_rec.agent       *
- *                                                                      *
- * Returns:                                                             *
- * -----------                                                          *
- * Boolean.  True if, yes this is a bookmark                            *
- *           False in all other cases. Including invalid args.          *
- *                                                                      *
- ************************************************************************/
-
-bool
-parse_is_bookmark(char *url, char *browser)
-{
-    char regex_favicon[MAX_RE_LENGTH] = "^\\/favicon\\.ico$";   /* Hold the FAVICON RE */
-    char regex_msie[MAX_RE_LENGTH] = " MSIE ";  /* Hold the MS IE RE */
-    char regex_exceptions[MAX_RE_LENGTH] = "(?:Opera|MSNc(?:OTH|IA))";  /* Hold the exceptions RE - Currently only Opera is falsely added. */
-
-    static pcre *cmp_regex_favicon = NULL;      /* FAVICON compiled RE */
-    static pcre *cmp_regex_msie = NULL;         /* MSIE compiled RE */
-    static pcre *cmp_regex_exceptions = NULL;   /* Exceptions compiled RE */
-
-    int ovector[OVECCOUNT];                     /* RE substring offsets array */
-    const char *error;                          /* RE error pointer, offset */
-    int erroffset;                              /* RE error value */
-    int str_length;
-    int rc;                                     /* RE Check return value */
-
-    /* Compile both RegEx's */
-    if (cmp_regex_favicon == NULL) {
-        cmp_regex_favicon = pcre_compile(regex_favicon, 0, &error, &erroffset, NULL);
-        VPRINT(VERBOSE2, "PCRE: Compile FAVICON%s", "\n")
-            if (cmp_regex_favicon == NULL) {
-            ERRVPRINT(VERBOSE0, "%s\n", _("Error: Failed to compile FAVICON RegEx!"));
-            exit(1);
-        }
-
-        cmp_regex_msie = pcre_compile(regex_msie, 0, &error, &erroffset, NULL);
-        VPRINT(VERBOSE2, "PCRE: Compile MSIE%s", "\n")
-            if (cmp_regex_favicon == NULL) {
-            ERRVPRINT(VERBOSE0, "%s\n", _("Error: Failed to compile MSIE RegEx!"));
-            exit(1);
-        }
-
-        cmp_regex_exceptions = pcre_compile(regex_exceptions, 0, &error, &erroffset, NULL);
-        VPRINT(VERBOSE2, "PCRE: Compile Exceptions%s", "\n")
-            if (cmp_regex_favicon == NULL) {
-            ERRVPRINT(VERBOSE0, "%s\n", _("Error: Failed to compile Exceptions RegEx!"));
-            exit(1);
-        }
-    }
-
-    str_length = (int) strlen(url);
-    rc = pcre_exec(cmp_regex_favicon, NULL, url, str_length, 0, 0, ovector, OVECCOUNT);
-    /* check for RE matching */
-    if (rc >= 0) {
-        /* Have matched! */
-        str_length = (int) strlen(browser);
-        rc = pcre_exec(cmp_regex_msie, NULL, browser, str_length, 0, 0, ovector, OVECCOUNT);
-        if (rc >= 0) {                          /* Have found "MSIE" now look for exceptions to reject */
-            rc = pcre_exec(cmp_regex_exceptions, NULL, browser, str_length, 0, 0, ovector, OVECCOUNT);
-            if (rc >= 0) {
-                return (false);                 /* Successfully matched an exception - hence failure */
-            } else {
-                return (true);
-            }
-        }                                       /* Don't bother with normal RE failure checks - too many false alarms */
-    }
-
-    return (false);
-}                                               /* parse_is_bookmark */
-
 /************************************************************************
  * identify_log_format                                                  *
  *                                                                      *
@@ -634,7 +551,7 @@ parse_is_bookmark(char *url, char *browser)
  *                                                                      *
  * Requires a line of the log to attempt to process                     *
  ************************************************************************/
-int
+static int
 identify_log_format(char *buffer)
 {
     int ovector[OVECCOUNT];                     /* RE substring offsets array */
@@ -698,7 +615,7 @@ identify_log_format(char *buffer)
  * Assigns the RE's to the various globals:                             *
  *   cmp_log_regexp_*                                                   *
  ************************************************************************/
-void
+static void
 re_compile_all_regexes(void)
 {
     char log_regexp_clf[MAX_RE_LENGTH] = PATTERN_CLF;
@@ -713,43 +630,43 @@ re_compile_all_regexes(void)
 
     /* CLF */
     cmp_log_regexp_clf = pcre_compile(log_regexp_clf, 0, &error, &erroffset, NULL);
-    VPRINT(VERBOSE2, "PCRE: Compile CLF%s", "\n")
-        if (cmp_log_regexp_clf == NULL) {
+    VPRINT(VERBOSE2, "PCRE: Compile CLF%s", "\n");
+    if (cmp_log_regexp_clf == NULL) {
         re_compile_failed(erroffset, error, log_regexp_clf);
     }
 
     /* Combined */
     cmp_log_regexp_combined = pcre_compile(log_regexp_combined, 0, &error, &erroffset, NULL);
-    VPRINT(VERBOSE2, "PCRE: Compile COMBINED%s", "\n")
-        if (cmp_log_regexp_combined == NULL) {
+    VPRINT(VERBOSE2, "PCRE: Compile COMBINED%s", "\n");
+    if (cmp_log_regexp_combined == NULL) {
         re_compile_failed(erroffset, error, log_regexp_combined);
     }
 
     /* Enhanced Combined */
     cmp_log_regexp_combined_enhanced = pcre_compile(log_regexp_combined_enhanced, 0, &error, &erroffset, NULL);
-    VPRINT(VERBOSE2, "PCRE: Compile COMBINED_ENHANCED%s", "\n")
-        if (cmp_log_regexp_combined_enhanced == NULL) {
+    VPRINT(VERBOSE2, "PCRE: Compile COMBINED_ENHANCED%s", "\n");
+    if (cmp_log_regexp_combined_enhanced == NULL) {
         re_compile_failed(erroffset, error, log_regexp_combined_enhanced);
     }
 
     /* FTP XFERLOG */
     cmp_log_regexp_xferlog = pcre_compile(log_regexp_xferlog, 0, &error, &erroffset, NULL);
-    VPRINT(VERBOSE2, "PCRE: Compile PATTERN_XFERLOG%s", "\n")
-        if (cmp_log_regexp_xferlog == NULL) {
+    VPRINT(VERBOSE2, "PCRE: Compile PATTERN_XFERLOG%s", "\n");
+    if (cmp_log_regexp_xferlog == NULL) {
         re_compile_failed(erroffset, error, log_regexp_xferlog);
     }
 
     /* SQUID LOG */
     cmp_log_regexp_squid = pcre_compile(log_regexp_squid, 0, &error, &erroffset, NULL);
-    VPRINT(VERBOSE2, "PCRE: Compile PATTERN_SQUID%s", "\n")
-        if (cmp_log_regexp_squid == NULL) {
+    VPRINT(VERBOSE2, "PCRE: Compile PATTERN_SQUID%s", "\n");
+    if (cmp_log_regexp_squid == NULL) {
         re_compile_failed(erroffset, error, log_regexp_squid);
     }
 
     /* DOMINO LOG */
     cmp_log_regexp_domino = pcre_compile(log_regexp_domino, 0, &error, &erroffset, NULL);
-    VPRINT(VERBOSE2, "PCRE: Compile PATTERN_COMBINED_DOMINO%s", "\n")
-        if (cmp_log_regexp_domino == NULL) {
+    VPRINT(VERBOSE2, "PCRE: Compile PATTERN_COMBINED_DOMINO%s", "\n");
+    if (cmp_log_regexp_domino == NULL) {
         re_compile_failed(erroffset, error, log_regexp_domino);
     }
 }
@@ -762,14 +679,14 @@ re_compile_all_regexes(void)
  * err: value returned from pcre_exec                                   *
  * str_ptr: String that failed to match                                 *
  ************************************************************************/
-void
+static void
 re_check_errors(int err)
 {
 
     /* Matching failed: handle error cases */
     switch (err) {
     case PCRE_ERROR_NOMATCH:
-        ERRVPRINT(VERBOSE1, "%s", _("Warning: No Regular Expression Match."));
+        ERRVPRINT(VERBOSE1, "%s", _("Warning: No Regular Expression Match. "));
         break;
         /*  Leave out the more explicit failure messages - we show the number, so can be found.
            case PCRE_ERROR_NULL:
@@ -800,7 +717,7 @@ re_check_errors(int err)
  * Display a failed RE Compile & where                                  *
  * FATAL failure. Will exit the run.                                    *
  ************************************************************************/
-void
+static void
 re_compile_failed(int err, const char *err_offset, char *re_str)
 {
     ERRVPRINT(VERBOSE0, "%s %d %s\n", _("FATAL ERROR! PCRE compilation failed at offset"), err, err_offset);
@@ -815,7 +732,7 @@ re_compile_failed(int err, const char *err_offset, char *re_str)
  * Display a failed substring extraction                                *
  * Error Only, as this should have failed the RE                        *
  ************************************************************************/
-void
+static void
 error_substring_extract(int err, int substr_idx)
 {
     ERRVPRINT(VERBOSE1, "%s %d\n", _("Error: Failed to extract substring:"), substr_idx);
