@@ -1,10 +1,10 @@
 /*
     AWFFull - A Webalizer Fork, Full o' features
     
-    $Id: preserve.c 350 2007-01-19 09:20:00Z steve $
+    $Id: preserve.c 438 2007-07-31 10:14:43Z steve $
 
     Copyright (C) 1997-2001  Bradford L. Barrett (brad@mrunix.net)
-    Copyright (C) 2004, 2005, 2006 by Stephen McInerney (spm@stedee.id.au)
+    Copyright (C) 2004, 2005, 2006, 2007 by Stephen McInerney (spm@stedee.id.au)
     Copyright (C) 2006 by Alexander Lazic (al-awffull@none.at)
 
     This program is free software; you can redistribute it and/or modify
@@ -30,22 +30,18 @@
     most recent version of the library and supporting documentation.
 */
 
-/*********************************************/
-/* STANDARD INCLUDES                         */
-/*********************************************/
-
 #include "awffull.h"                            /* main header              */
 
 /* local variables */
 struct history history_list[MAXHISTLEN];        /* Complete Array for the history */
 
 int hist_month[12], hist_year[12];              /* arrays for monthly total */
-u_long hist_hit[12];                            /* calculations: used to    */
-u_long hist_files[12];                          /* produce index.html       */
-u_long hist_site[12];                           /* these are read and saved */
+unsigned long hist_hit[12];                     /* calculations: used to    */
+unsigned long hist_files[12];                   /* produce index.html       */
+unsigned long hist_site[12];                    /* these are read and saved */
 double hist_xfer[12];                           /* in the history file      */
-u_long hist_page[12];
-u_long hist_visit[12];
+unsigned long hist_page[12];
+unsigned long hist_visit[12];
 
 int hist_fday[12], hist_lday[12];               /* first/last day arrays    */
 
@@ -80,7 +76,6 @@ cmp_history(const void *h1, const void *h2)
 /*********************************************
  * GET_HISTORY - load in history file        *
  *********************************************/
-
 void
 get_history()
 {
@@ -88,7 +83,7 @@ get_history()
     int numfields = 0;                          /* Number of fields returned from each line */
 
     /*  Used to verify if an older history file */
-    int nbr_hist_entries = 0;                   /* How many Lines out of the hitsory file have been read */
+    int nbr_hist_entries = 0;                   /* How many lines out of the history file have been read */
 
     FILE *hist_fp;
     char buffer[BUFSIZE];
@@ -108,9 +103,9 @@ get_history()
     }
 
     /* Open History file and Process */
-    hist_fp = fopen(hist_fname, "r");
+    hist_fp = fopen(g_settings.settings.history_filename, "r");
     if (hist_fp) {
-        VPRINT(VERBOSE1, "%s %s\n", _("Reading history file..."), hist_fname);
+        VPRINT(VERBOSE1, "%s %s\n", _("Reading history file..."), g_settings.settings.history_filename);
         while ((fgets(buffer, BUFSIZE, hist_fp)) != NULL) {
             /* month# year# requests files sites xfer firstday lastday */
             numfields = sscanf(buffer, "%d %d %lu %lu %lu %lf %d %d %lu %lu",
@@ -144,14 +139,13 @@ get_history()
 /*********************************************
  * PUT_HISTORY - write out history file      *
  *********************************************/
-
 void
 put_history()
 {
     int i;
     FILE *hist_fp;
 
-    hist_fp = fopen(hist_fname, "w");
+    hist_fp = fopen(g_settings.settings.history_filename, "w");
 
     if (hist_fp) {
         VPRINT(VERBOSE1, "%s\n", _("Saving history information..."));
@@ -164,14 +158,13 @@ put_history()
         }
         fclose(hist_fp);
     } else {
-        ERRVPRINT(VERBOSE1, "%s %s\n", _("Error: Unable to write history file"), hist_fname);
+        ERRVPRINT(VERBOSE1, "%s %s\n", _("Error: Unable to write history file"), g_settings.settings.history_filename);
     }
 }
 
 /*********************************************/
 /* SAVE_STATE - save internal data structs   */
 /*********************************************/
-
 int
 save_state()
 {
@@ -189,7 +182,7 @@ save_state()
     char buffer[BUFSIZE];
 
     /* Open data file for write */
-    fp = fopen(state_fname, "w");
+    fp = fopen(g_settings.settings.state_filename, "w");
     if (fp == NULL)
         return 1;
 
@@ -210,26 +203,29 @@ save_state()
         return 1;                               /* error exit */
 
     /* Monthly totals for sites, urls, etc... */
-    snprintf(buffer, sizeof(buffer), "%lu %lu %lu %lu %lu %lu %llu %lu %lu %lu %lu %lu %lu\n", t_hit, t_file, t_site, t_url, t_ref, t_agent, t_xfer, t_page, t_visit, t_user,
-             t_bookmarks, t_bad, t_ignored);
+    snprintf(buffer, sizeof(buffer), "%lu %lu %lu %lu %lu %lu %llu %lu %lu %lu %d %lu %lu\n", g_counters.month.hit, g_counters.month.file, g_counters.month.site,
+             g_counters.month.url, g_counters.month.ref, g_counters.month.agent, g_counters.month.vol, g_counters.month.page, g_counters.month.visit, g_counters.month.user, 0,
+             g_counters.generic.bad, g_counters.generic.ignored);
     if (fputs(buffer, fp) == EOF)
         return 1;                               /* error exit */
 
     /* Daily totals for sites, urls, etc... */
-    snprintf(buffer, sizeof(buffer), "%lu %lu %lu %d %d\n", dt_site, ht_hit, mh_hit, f_day, l_day);
+    snprintf(buffer, sizeof(buffer), "%u %lu %lu %d %d\n", 0, ht_hit, mh_hit, f_day, l_day);
     if (fputs(buffer, fp) == EOF)
         return 1;                               /* error exit */
 
     /* Monthly (by day) total array */
     for (i = 0; i < 31; i++) {
-        snprintf(buffer, sizeof(buffer), "%lu %lu %llu %lu %lu %lu %lu\n", tm_hit[i], tm_file[i], tm_xfer[i], tm_site[i], tm_page[i], tm_visit[i], tm_bookm[i]);
+        snprintf(buffer, sizeof(buffer), "%lu %lu %llu %lu %lu %lu %d\n", g_counters.day.hit[i], g_counters.day.file[i], g_counters.day.vol[i], g_counters.day.site[i],
+                 g_counters.day.page[i], g_counters.day.visit[i], 0);
         if (fputs(buffer, fp) == EOF)
             return 1;                           /* error exit */
     }
 
     /* Daily (by hour) total array */
     for (i = 0; i < 24; i++) {
-        snprintf(buffer, sizeof(buffer), "%lu %lu %llu %lu %lu\n", th_hit[i], th_file[i], th_xfer[i], th_page[i], th_bookm[i]);
+        snprintf(buffer, sizeof(buffer), "%lu %lu %llu %lu %lu\n", g_counters.hour.hit[i], g_counters.hour.file[i], g_counters.hour.vol[i], g_counters.hour.page[i],
+                 g_counters.hour.site[i]);
         if (fputs(buffer, fp) == EOF)
             return 1;                           /* error exit */
     }
@@ -248,8 +244,8 @@ save_state()
     for (i = 0; i < MAXHASH; i++) {
         uptr = um_htab[i];
         while (uptr != NULL) {
-            snprintf(buffer, sizeof(buffer), "%s\n%d %lu %lu %llu %lu %lu %lu %llu\n", uptr->string, uptr->flag, uptr->count, uptr->files, uptr->xfer, uptr->entry, uptr->exit,
-                     uptr->pcount, uptr->pxfer);
+            snprintf(buffer, sizeof(buffer), "%s\n%d %lu %lu %llu %lu %lu %lu %llu %lu\n", uptr->string, uptr->flag, uptr->count, uptr->files, uptr->xfer, uptr->entry, uptr->exit,
+                     uptr->pcount, uptr->pxfer, uptr->single_access);
             if (fputs(buffer, fp) == EOF)
                 return 1;
             uptr = uptr->next;
@@ -265,8 +261,8 @@ save_state()
     for (i = 0; i < MAXHASH; i++) {
         hptr = sm_htab[i];
         while (hptr != NULL) {
-            snprintf(buffer, sizeof(buffer), "%s\n%d %lu %lu %llu %lu %lu %lu\n%s\n", hptr->string, hptr->flag, hptr->count, hptr->files, hptr->xfer, hptr->visit, hptr->pages,
-                     hptr->tstamp, (hptr->lasturl == blank_str) ? "-" : hptr->lasturl);
+            snprintf(buffer, sizeof(buffer), "%s\n%d %lu %lu %llu %lu %lu %lu %d\n%s\n", hptr->string, hptr->flag, hptr->count, hptr->files, hptr->xfer, hptr->visit, hptr->pages,
+                     hptr->tstamp, hptr->lasturl_is_entry, (hptr->lasturl == blank_str) ? "-" : hptr->lasturl);
             if (fputs(buffer, fp) == EOF)
                 return 1;                       /* error exit */
             hptr = hptr->next;
@@ -281,8 +277,8 @@ save_state()
     for (i = 0; i < MAXHASH; i++) {
         hptr = sd_htab[i];
         while (hptr != NULL) {
-            snprintf(buffer, sizeof(buffer), "%s\n%d %lu %lu %llu %lu %lu\n%s\n", hptr->string, hptr->flag, hptr->count, hptr->files, hptr->xfer, hptr->visit, hptr->tstamp,
-                     (hptr->lasturl == blank_str) ? "-" : hptr->lasturl);
+            snprintf(buffer, sizeof(buffer), "%s\n%d %lu %lu %llu %lu %lu %lu %d\n%s\n", hptr->string, hptr->flag, hptr->count, hptr->files, hptr->xfer, hptr->visit, hptr->pages,
+                     hptr->tstamp, hptr->lasturl_is_entry, (hptr->lasturl == blank_str) ? "-" : hptr->lasturl);
             if (fputs(buffer, fp) == EOF)
                 return 1;
             hptr = hptr->next;
@@ -294,7 +290,7 @@ save_state()
     /* Referrer list */
     if (fputs("# -referrers- \n", fp) == EOF)
         return 1;                               /* error exit */
-    if (t_ref != 0) {
+    if (g_counters.month.ref != 0) {
         for (i = 0; i < MAXHASH; i++) {
             rptr = rm_htab[i];
             while (rptr != NULL) {
@@ -311,7 +307,7 @@ save_state()
     /* User agent list */
     if (fputs("# -agents- \n", fp) == EOF)
         return 1;                               /* error exit */
-    if (t_agent != 0) {
+    if (g_counters.month.agent != 0) {
         for (i = 0; i < MAXHASH; i++) {
             aptr = am_htab[i];
             while (aptr != NULL) {
@@ -394,11 +390,11 @@ restore_state()
     struct enode t_enode;
 
     char buffer[BUFSIZE];
-    char tmp_buf[BUFSIZE], tmp2_buf[BUFSIZE];
+    char buffer2[BUFSIZE], buffer3[BUFSIZE];
 
     struct tm time_rec;                         /* Gotta convert that string'ed time into a timerec first */
 
-    u_long ul_bogus = 0;
+    unsigned long ul_bogus = 0;
 
     unsigned int tmp_version_major = 0;         /* Scan in the version of the Current Run Data file. Use int's as a float scan was not consistant */
     unsigned int tmp_version_minor = 0;
@@ -406,7 +402,10 @@ restore_state()
     unsigned long rundata_version = 0;          /* The webalizer version from the file header */
     int ssout;                                  /* sscanf return result for checking */
 
-    fp = fopen(state_fname, "r");
+    unsigned long ignore_this_value = 0;        /* Ignore this value - was old save, now dropped */
+    bool ignore_this_bool = false;
+
+    fp = fopen(g_settings.settings.state_filename, "r");
     if (fp == NULL) {
         /* Previous run data not found... */
         VPRINT(VERBOSE1, "%s\n", _("Previous run data not found..."));
@@ -414,10 +413,11 @@ restore_state()
     }
 
     /* Reading previous run data... */
-    VPRINT(VERBOSE1, "%s %s\n", _("Reading previous run data.."), state_fname);
+    VPRINT(VERBOSE1, "%s %s\n", _("Reading previous run data.."), g_settings.settings.state_filename);
 
     /* get easy stuff */
-    if ((fgets(buffer, BUFSIZE, fp)) != NULL) { /* Header record */
+    /* Header record */
+    if ((fgets(buffer, BUFSIZE, fp)) != NULL) {
         ssout = sscanf(buffer, "# AWFFull V%u.%u.%u", &tmp_version_major, &tmp_version_minor, &tmp_version_micro);
         if (ssout <= 0) {
             /* Possibly Webalizer format header? */
@@ -446,67 +446,75 @@ restore_state()
         }
     } /* bad magic? */
     else
-        return 1;                               /* error exit */
+        return 98;                              /* error exit */
 
-    /* Get current timestamp */
+    /* Current date/time          */
     if ((fgets(buffer, BUFSIZE, fp)) != NULL) {
         memset(&time_rec, 0, sizeof(time_rec));
         strptime(buffer, "%Y %m %d %H %M %S", &time_rec);
         /* calculate current timestamp (seconds since epoch) and setup current time globals */
         time_rec.tm_isdst = -1;                 /* stop mktime from resetting for daylight savings */
         cur_tstamp = mktime(&time_rec);
-        VPRINT(VERBOSE1, "%s %lu\n", _("Previous Final Timestamp:"), cur_tstamp);
+        VPRINT(VERBOSE1, "%s %lu", _("Previous Final Timestamp:"), cur_tstamp);
         g_cur_year = time_rec.tm_year + 1900;
         g_cur_month = time_rec.tm_mon + 1;
         g_cur_day = time_rec.tm_mday;
         g_cur_hour = time_rec.tm_hour;
         g_cur_min = time_rec.tm_min;
         g_cur_sec = time_rec.tm_sec;
+        VPRINT(VERBOSE1, " %d-%d-%d:%d:%d:%d\n", g_cur_year, g_cur_month, g_cur_day, g_cur_hour, g_cur_min, g_cur_sec);
     } else
         return 2;                               /* error exit */
 
-    /* Get monthly totals */
+    /* Monthly totals for sites, urls, etc... */
     if ((fgets(buffer, BUFSIZE, fp)) != NULL) {
         if (rundata_version < 302) {
-            sscanf(buffer, "%lu %lu %lu %lu %lu %lu %llu %lu %lu %lu", &t_hit, &t_file, &t_site, &t_url, &t_ref, &t_agent, &t_xfer, &t_page, &t_visit, &t_user);
+            sscanf(buffer, "%lu %lu %lu %lu %lu %lu %llu %lu %lu %lu", &g_counters.month.hit, &g_counters.month.file, &g_counters.month.site, &g_counters.month.url,
+                   &g_counters.month.ref, &g_counters.month.agent, &g_counters.month.vol, &g_counters.month.page, &g_counters.month.visit, &g_counters.month.user);
         } else {
-            sscanf(buffer, "%lu %lu %lu %lu %lu %lu %llu %lu %lu %lu %lu %lu %lu", &t_hit, &t_file, &t_site, &t_url, &t_ref, &t_agent, &t_xfer, &t_page, &t_visit, &t_user,
-                   &t_bookmarks, &t_bad, &t_ignored);
+            sscanf(buffer, "%lu %lu %lu %lu %lu %lu %llu %lu %lu %lu %lu %lu %lu", &g_counters.month.hit, &g_counters.month.file, &g_counters.month.site, &g_counters.month.url,
+                   &g_counters.month.ref, &g_counters.month.agent, &g_counters.month.vol, &g_counters.month.page, &g_counters.month.visit, &g_counters.month.user,
+                   &ignore_this_value, &g_counters.generic.bad, &g_counters.generic.ignored);
         }
     } else
         return 3;                               /* error exit */
 
-    /* Get daily totals */
+    /* Daily totals for sites, urls, etc... */
     if ((fgets(buffer, BUFSIZE, fp)) != NULL) {
-        sscanf(buffer, "%lu %lu %lu %d %d", &dt_site, &ht_hit, &mh_hit, &f_day, &l_day);
+        sscanf(buffer, "%lu %lu %lu %d %d", &ignore_this_value, &ht_hit, &mh_hit, &f_day, &l_day);
     } else
         return 4;                               /* error exit */
 
-    /* get daily totals */
+    /* Monthly (by day) total array */
     for (i = 0; i < 31; i++) {
         if ((fgets(buffer, BUFSIZE, fp)) != NULL) {
             if (rundata_version < 302) {
-                sscanf(buffer, "%lu %lu %llu %lu %lu %lu", &tm_hit[i], &tm_file[i], &tm_xfer[i], &tm_site[i], &tm_page[i], &tm_visit[i]);
+                sscanf(buffer, "%lu %lu %llu %lu %lu %lu", &g_counters.day.hit[i], &g_counters.day.file[i], &g_counters.day.vol[i], &g_counters.day.site[i],
+                       &g_counters.day.page[i], &g_counters.day.visit[i]);
             } else {
-                sscanf(buffer, "%lu %lu %llu %lu %lu %lu %lu", &tm_hit[i], &tm_file[i], &tm_xfer[i], &tm_site[i], &tm_page[i], &tm_visit[i], &tm_bookm[i]);
+                sscanf(buffer, "%lu %lu %llu %lu %lu %lu %lu", &g_counters.day.hit[i], &g_counters.day.file[i], &g_counters.day.vol[i], &g_counters.day.site[i],
+                       &g_counters.day.page[i], &g_counters.day.visit[i], &ignore_this_value);
             }
         } else
             return 5;                           /* error exit */
     }
 
-    /* get hourly totals */
+    /* Daily (by hour) total array */
     for (i = 0; i < 24; i++) {
         if ((fgets(buffer, BUFSIZE, fp)) != NULL) {
             if (rundata_version < 302) {
-                sscanf(buffer, "%lu %lu %llu %lu", &th_hit[i], &th_file[i], &th_xfer[i], &th_page[i]);
+                sscanf(buffer, "%lu %lu %llu %lu", &g_counters.hour.hit[i], &g_counters.hour.file[i], &g_counters.hour.vol[i], &g_counters.hour.page[i]);
+            } else if (rundata_version < 30801) {
+                sscanf(buffer, "%lu %lu %llu %lu %lu", &g_counters.hour.hit[i], &g_counters.hour.file[i], &g_counters.hour.vol[i], &g_counters.hour.page[i], &ignore_this_value);
             } else {
-                sscanf(buffer, "%lu %lu %llu %lu %lu", &th_hit[i], &th_file[i], &th_xfer[i], &th_page[i], &th_bookm[i]);
+                sscanf(buffer, "%lu %lu %llu %lu %lu", &g_counters.hour.hit[i], &g_counters.hour.file[i], &g_counters.hour.vol[i], &g_counters.hour.page[i],
+                       &g_counters.hour.site[i]);
             }
         } else
             return 6;                           /* error exit */
     }
 
-    /* get response code totals */
+    /* Response codes */
     for (i = 0; i < TOTAL_RC; i++) {
         if ((fgets(buffer, BUFSIZE, fp)) != NULL)
             sscanf(buffer, "%lu", &response[i].count);
@@ -514,6 +522,10 @@ restore_state()
             return 7;                           /* error exit */
     }
 
+
+    /* now we need to restore our linked lists */
+    /* URL list */
+
     /* Kludge for V2.01-06 TOTAL_RC off by one bug */
     if (!strncmp(buffer, "# -urls- ", 9))
         response[TOTAL_RC - 1].count = 0;
@@ -523,25 +535,31 @@ restore_state()
         /* url table */
         if ((fgets(buffer, BUFSIZE, fp)) != NULL) {     /* Table header */
             if (strncmp(buffer, "# -urls- ", 9))
-                return 10;
+                return 8;
         } /* (url)        */
         else
-            return 10;                          /* error exit */
+            return 8;                           /* error exit */
     }
 
-    while ((fgets(buffer, BUFSIZE, fp)) != NULL) {
-        if (!strncmp(buffer, "# End Of Table ", 15))
+    while ((fgets(buffer2, BUFSIZE, fp)) != NULL) {
+        if (!strncmp(buffer2, "# End Of Table ", 15))
             break;
-        strncpy(tmp_buf, buffer, MAXURLH);
-        tmp_buf[strlen(tmp_buf) - 1] = 0;
+        buffer2[strlen(buffer2) - 1] = '\0';    /* Chop off trailing newline */
 
         if ((fgets(buffer, BUFSIZE, fp)) == NULL)
-            return 10;                          /* error exit */
+            return 9;                           /* error exit */
         if (!isdigit((int) buffer[0]))
-            return 10;                          /* error exit */
+            return 9;                           /* error exit */
 
+        /* Zero values not used in loads of older data */
+        t_unode.single_access = 0;
+        t_unode.pcount = 0;
+        t_unode.pxfer = 0;
         /* load temporary node data */
-        if (rundata_version > 30403) {
+        if (rundata_version >= 30801) {
+            sscanf(buffer, "%d %lu %lu %llu %lu %lu %lu %llu %lu", &t_unode.flag, &t_unode.count, &t_unode.files, &t_unode.xfer, &t_unode.entry, &t_unode.exit, &t_unode.pcount,
+                   &t_unode.pxfer, &t_unode.single_access);
+        } else if (rundata_version > 30403) {
             sscanf(buffer, "%d %lu %lu %llu %lu %lu %lu %llu", &t_unode.flag, &t_unode.count, &t_unode.files, &t_unode.xfer, &t_unode.entry, &t_unode.exit, &t_unode.pcount,
                    &t_unode.pxfer);
         } else {
@@ -549,43 +567,48 @@ restore_state()
         }
 
         /* Good record, insert into hash table */
-        if (put_unode(tmp_buf, t_unode.flag, t_unode.count, t_unode.xfer, &ul_bogus, t_unode.entry, t_unode.exit, um_htab)) {
+        if (put_unode(buffer2, t_unode.flag, t_unode.count, t_unode.xfer, &ul_bogus, t_unode.entry, t_unode.exit, t_unode.single_access, 0, um_htab)) {
             /* Error adding URL node, skipping ... */
             ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding URL node, skipping"), t_unode.string);
         }
     }
 
-    /* monthly sites table */
+    /* daily hostname list */
     if ((fgets(buffer, BUFSIZE, fp)) != NULL) { /* Table header */
         if (strncmp(buffer, "# -sites- ", 10))
-            return 8;
+            return 10;
     } /* (monthly)    */
     else
-        return 8;                               /* error exit */
+        return 10;                              /* error exit */
 
-    while ((fgets(buffer, BUFSIZE, fp)) != NULL) {
+    while ((fgets(buffer2, BUFSIZE, fp)) != NULL) {
         /* Check for end of table */
-        if (!strncmp(buffer, "# End Of Table ", 15))
+        if (!strncmp(buffer2, "# End Of Table ", 15))
             break;
-        strncpy(tmp_buf, buffer, MAXHOST);
-        tmp_buf[strlen(buffer) - 1] = 0;
+        buffer2[strlen(buffer2) - 1] = '\0';    /* Chop off trailing newline */
 
         if ((fgets(buffer, BUFSIZE, fp)) == NULL)
-            return 8;                           /* error exit */
+            return 11;                          /* error exit */
         if (!isdigit((int) buffer[0]))
-            return 8;                           /* error exit */
+            return 11;                          /* error exit */
 
+        /* Zero values not used in loads of older data */
+        t_hnode.pages = 0;
+        t_hnode.lasturl_is_entry = false;
         /* load temporary node data */
         if (rundata_version < 300) {
             sscanf(buffer, "%d %lu %lu %llu %lu %lu", &t_hnode.flag, &t_hnode.count, &t_hnode.files, &t_hnode.xfer, &t_hnode.visit, &t_hnode.tstamp);
             t_hnode.pages = 0;
-        } else {                                /* version 3.00, aka 300, included t_hnode.pages in the output */
+        } else if (rundata_version < 30801) {
             sscanf(buffer, "%d %lu %lu %llu %lu %lu %lu", &t_hnode.flag, &t_hnode.count, &t_hnode.files, &t_hnode.xfer, &t_hnode.visit, &t_hnode.pages, &t_hnode.tstamp);
+        } else {                                /* version 3.00, aka 300, included t_hnode.pages in the output */
+            sscanf(buffer, "%d %lu %lu %llu %lu %lu %lu %hhd", &t_hnode.flag, &t_hnode.count, &t_hnode.files, &t_hnode.xfer, &t_hnode.visit, &t_hnode.pages, &t_hnode.tstamp,
+                   &t_hnode.lasturl_is_entry);
         }
 
         /* get last url */
         if ((fgets(buffer, BUFSIZE, fp)) == NULL)
-            return 8;                           /* error exit */
+            return 11;                          /* error exit */
         if (buffer[0] == '-')
             t_hnode.lasturl = blank_str;
         else {
@@ -594,40 +617,51 @@ restore_state()
         }
 
         /* Good record, insert into hash table */
-        /* FIXME!!! */
         if (put_hnode
-            (tmp_buf, t_hnode.flag, t_hnode.count, t_hnode.files, t_hnode.xfer, &ul_bogus, t_hnode.visit + 1, t_hnode.pages, t_hnode.tstamp, t_hnode.lasturl, sm_htab, false)) {
+            (buffer2, t_hnode.flag, t_hnode.count, t_hnode.files, t_hnode.xfer, &ul_bogus, t_hnode.visit, t_hnode.pages, t_hnode.tstamp, "", t_hnode.lasturl, sm_htab,
+             t_hnode.lasturl_is_entry, false, &ignore_this_bool)) {
             /* Error adding host node (monthly), skipping .... */
             ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Host node (monthly), skipping"), t_hnode.string);
         }
     }
 
-    /* Daily sites table */
+    /* hourly hostname list */
     if ((fgets(buffer, BUFSIZE, fp)) != NULL) { /* Table header */
         if (strncmp(buffer, "# -sites- ", 10))
-            return 9;
+            return 12;
     } /* (daily)      */
     else
-        return 9;                               /* error exit */
+        return 12;                              /* error exit */
 
-    while ((fgets(buffer, BUFSIZE, fp)) != NULL) {
+    while ((fgets(buffer2, BUFSIZE, fp)) != NULL) {
         /* Check for end of table */
-        if (!strncmp(buffer, "# End Of Table ", 15))
+        if (!strncmp(buffer2, "# End Of Table ", 15))
             break;
-        strncpy(tmp_buf, buffer, MAXHOST);
-        tmp_buf[strlen(buffer) - 1] = 0;
+        buffer2[strlen(buffer2) - 1] = '\0';    /* Chop off trailing newline */
 
         if ((fgets(buffer, BUFSIZE, fp)) == NULL)
-            return 9;                           /* error exit */
+            return 13;                          /* error exit */
         if (!isdigit((int) buffer[0]))
-            return 9;                           /* error exit */
+            return 13;                          /* error exit */
 
+        /* Zero values not used in loads of older data */
+        t_hnode.pages = 0;
+        t_hnode.lasturl_is_entry = false;
         /* load temporary node data */
-        sscanf(buffer, "%d %lu %lu %llu %lu %lu", &t_hnode.flag, &t_hnode.count, &t_hnode.files, &t_hnode.xfer, &t_hnode.visit, &t_hnode.tstamp);
+        if (rundata_version < 300) {
+            sscanf(buffer, "%d %lu %lu %llu %lu %lu", &t_hnode.flag, &t_hnode.count, &t_hnode.files, &t_hnode.xfer, &t_hnode.visit, &t_hnode.tstamp);
+            t_hnode.pages = 0;
+        } else if (rundata_version < 30801) {   /* version 3.00, aka 300, included t_hnode.pages in the output */
+            sscanf(buffer, "%d %lu %lu %llu %lu %lu %lu", &t_hnode.flag, &t_hnode.count, &t_hnode.files, &t_hnode.xfer, &t_hnode.visit, &t_hnode.pages, &t_hnode.tstamp);
+        } else {                                /* version 3.8.1, aka 30801, included t_hnode.lasturl_is_entry in the output */
+            sscanf(buffer, "%d %lu %lu %llu %lu %lu %lu %hhd", &t_hnode.flag, &t_hnode.count, &t_hnode.files, &t_hnode.xfer, &t_hnode.visit, &t_hnode.pages, &t_hnode.tstamp,
+                   &(t_hnode.lasturl_is_entry));
+        }
+
 
         /* get last url */
         if ((fgets(buffer, BUFSIZE, fp)) == NULL)
-            return 9;                           /* error exit */
+            return 13;                          /* error exit */
         if (buffer[0] == '-')
             t_hnode.lasturl = blank_str;
         else {
@@ -636,93 +670,88 @@ restore_state()
         }
 
         /* Good record, insert into hash table */
-        /* FIXME!!! */
-        if (put_hnode(tmp_buf, t_hnode.flag, t_hnode.count, t_hnode.files, t_hnode.xfer, &ul_bogus, t_hnode.visit + 1, 0, t_hnode.tstamp, t_hnode.lasturl, sd_htab, false)) {
+        if (put_hnode
+            (buffer2, t_hnode.flag, t_hnode.count, t_hnode.files, t_hnode.xfer, &ul_bogus, t_hnode.visit, t_hnode.pages, t_hnode.tstamp, "", t_hnode.lasturl, sd_htab,
+             t_hnode.lasturl_is_entry, false, &ignore_this_bool)) {
             /* Error adding host node (daily), skipping .... */
             ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Host node (daily), skipping"), t_hnode.string);
         }
     }
 
-    /* Referrers table */
+    /* Referrer list */
     if ((fgets(buffer, BUFSIZE, fp)) != NULL) { /* Table header */
         if (strncmp(buffer, "# -referrers- ", 14))
-            return 11;
-    } /* (referrers) */
-    else
-        return 11;                              /* error exit */
+            return 14;
+    } else
+        return 14;                              /* error exit */
 
-    while ((fgets(buffer, BUFSIZE, fp)) != NULL) {
-        if (!strncmp(buffer, "# End Of Table ", 15))
+    while ((fgets(buffer2, BUFSIZE, fp)) != NULL) {
+        if (!strncmp(buffer2, "# End Of Table ", 15))
             break;
-        strncpy(tmp_buf, buffer, MAXREFH);
-        tmp_buf[strlen(buffer) - 1] = 0;
+        buffer2[strlen(buffer2) - 1] = '\0';    /* Chop off trailing newline */
 
         if ((fgets(buffer, BUFSIZE, fp)) == NULL)
-            return 11;                          /* error exit */
+            return 15;                          /* error exit */
         if (!isdigit((int) buffer[0]))
-            return 11;                          /* error exit */
+            return 15;                          /* error exit */
 
         /* load temporary node data */
         sscanf(buffer, "%d %lu", &t_rnode.flag, &t_rnode.count);
 
         /* insert node */
-        if (put_rnode(tmp_buf, t_rnode.flag, t_rnode.count, &ul_bogus, rm_htab)) {
-            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Referrer node, skipping"), log_rec.refer);
+        if (put_rnode(buffer2, t_rnode.flag, t_rnode.count, &ul_bogus, rm_htab)) {
+            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Referrer node, skipping"), buffer2);
         }
     }
 
-    /* Agents table */
+    /* User agent list */
     if ((fgets(buffer, BUFSIZE, fp)) != NULL) { /* Table header */
         if (strncmp(buffer, "# -agents- ", 11))
-            return 12;
-    } /* (agents) */
-    else
-        return 12;                              /* error exit */
+            return 16;
+    } else
+        return 16;                              /* error exit */
 
-    while ((fgets(buffer, BUFSIZE, fp)) != NULL) {
-        if (!strncmp(buffer, "# End Of Table ", 15))
+    while ((fgets(buffer2, BUFSIZE, fp)) != NULL) {
+        if (!strncmp(buffer2, "# End Of Table ", 15))
             break;
-        strncpy(tmp_buf, buffer, MAXAGENT);
-        tmp_buf[strlen(buffer) - 1] = 0;
+        buffer2[strlen(buffer2) - 1] = '\0';    /* Chop off trailing newline */
 
         if ((fgets(buffer, BUFSIZE, fp)) == NULL)
-            return 12;                          /* error exit */
+            return 17;                          /* error exit */
         if (!isdigit((int) buffer[0]))
-            return 12;                          /* error exit */
+            return 17;                          /* error exit */
 
         /* load temporary node data */
         sscanf(buffer, "%d %lu", &t_anode.flag, &t_anode.count);
 
         /* insert node */
-        if (put_anode(tmp_buf, t_anode.flag, t_anode.count, &ul_bogus, am_htab)) {
-            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding User Agent node, skipping"), log_rec.agent);
+        if (put_anode(buffer2, t_anode.flag, t_anode.count, &ul_bogus, am_htab)) {
+            ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding User Agent node, skipping"), buffer2);
         }
     }
 
     /* Search Strings table */
     if ((fgets(buffer, BUFSIZE, fp)) != NULL) { /* Table header */
         if (strncmp(buffer, "# -search string", 16))
-            return 13;
-    } /* (search) */
-    else
-        return 13;                              /* error exit */
+            return 18;
+    } else
+        return 18;                              /* error exit */
 
-    while ((fgets(buffer, BUFSIZE, fp)) != NULL) {
-        if (!strncmp(buffer, "# End Of Table ", 15))
+    while ((fgets(buffer2, BUFSIZE, fp)) != NULL) {
+        if (!strncmp(buffer2, "# End Of Table ", 15))
             break;
-        strncpy(tmp_buf, buffer, MAXSRCH);
-        tmp_buf[strlen(buffer) - 1] = 0;
+        buffer2[strlen(buffer2) - 1] = '\0';    /* Chop off trailing newline */
 
         if ((fgets(buffer, BUFSIZE, fp)) == NULL)
-            return 13;                          /* error exit */
+            return 19;                          /* error exit */
         if (!isdigit((int) buffer[0]))
-            return 13;                          /* error exit */
+            return 19;                          /* error exit */
 
         /* load temporary node data */
         sscanf(buffer, "%lu", &t_snode.count);
 
         /* insert node */
-        if (put_snode(tmp_buf, t_snode.count, sr_htab)) {
+        if (put_snode(buffer2, t_snode.count, sr_htab)) {
             ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Search String Node, skipping"), t_snode.string);
         }
     }
@@ -730,27 +759,26 @@ restore_state()
     /* usernames table */
     if ((fgets(buffer, BUFSIZE, fp)) != NULL) { /* Table header */
         if (strncmp(buffer, "# -usernames- ", 10))
-            return 14;
+            return 20;
     } else
-        return 14;                              /* error exit */
+        return 20;                              /* error exit */
 
-    while ((fgets(buffer, BUFSIZE, fp)) != NULL) {
+    while ((fgets(buffer2, BUFSIZE, fp)) != NULL) {
         /* Check for end of table */
-        if (!strncmp(buffer, "# End Of Table ", 15))
+        if (!strncmp(buffer2, "# End Of Table ", 15))
             break;
-        strncpy(tmp_buf, buffer, MAXIDENT);
-        tmp_buf[strlen(buffer) - 1] = 0;
+        buffer2[strlen(buffer2) - 1] = '\0';    /* Chop off trailing newline */
 
         if ((fgets(buffer, BUFSIZE, fp)) == NULL)
-            return 14;                          /* error exit */
+            return 21;                          /* error exit */
         if (!isdigit((int) buffer[0]))
-            return 14;                          /* error exit */
+            return 21;                          /* error exit */
 
         /* load temporary node data */
         sscanf(buffer, "%d %lu %lu %llu %lu %lu", &t_inode.flag, &t_inode.count, &t_inode.files, &t_inode.xfer, &t_inode.visit, &t_inode.tstamp);
 
         /* Good record, insert into hash table */
-        if (put_inode(tmp_buf, t_inode.flag, t_inode.count, t_inode.files, t_inode.xfer, &ul_bogus, t_inode.visit + 1, t_inode.tstamp, im_htab, false)) {
+        if (put_inode(buffer2, t_inode.flag, t_inode.count, t_inode.files, t_inode.xfer, &ul_bogus, t_inode.visit, t_inode.tstamp, im_htab, false)) {
             /* Error adding username node, skipping .... */
             ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding Username node, skipping"), t_inode.string);
         }
@@ -760,32 +788,29 @@ restore_state()
         /* 404 table */
         if ((fgets(buffer, BUFSIZE, fp)) != NULL) {     /* Table header */
             if (strncmp(buffer, "# -404errors- ", 14))
-                return 15;
+                return 22;
         } else
-            return 15;                          /* error exit */
+            return 22;                          /* error exit */
 
-        while ((fgets(buffer, BUFSIZE, fp)) != NULL) {
-            if (!strncmp(buffer, "# End Of Table ", 15))
+        while ((fgets(buffer2, BUFSIZE, fp)) != NULL) {
+            if (!strncmp(buffer2, "# End Of Table ", 15))
                 break;
-            strncpy(tmp_buf, buffer, MAXURLH);
-            tmp_buf[strlen(tmp_buf) - 1] = 0;
+            buffer2[strlen(buffer2) - 1] = '\0';        /* Chop off trailing newline */
 
             if ((fgets(buffer, BUFSIZE, fp)) == NULL)
-                return 15;                      /* error exit */
+                return 23;                      /* error exit */
             if (!isdigit((int) buffer[0]))
-                return 15;                      /* error exit */
+                return 23;                      /* error exit */
 
             /* load temporary node data */
             sscanf(buffer, "%lu", &t_enode.count);
 
-            if ((fgets(buffer, BUFSIZE, fp)) == NULL)
-                return 15;                      /* error exit */
-
-            strncpy(tmp2_buf, buffer, MAXURLH);
-            tmp2_buf[strlen(tmp2_buf) - 1] = 0;
+            if ((fgets(buffer3, BUFSIZE, fp)) == NULL)
+                return 23;                      /* error exit */
+            buffer3[strlen(buffer3) - 1] = '\0';        /* Chop off trailing newline */
 
             /* Good record, insert into hash table */
-            if (put_enode(tmp_buf, tmp2_buf, OBJ_REG, t_enode.count, &t_errorp, ep_htab)) {
+            if (put_enode(buffer2, buffer3, OBJ_REG, t_enode.count, &g_counters.generic.error, ep_htab)) {
                 /* Error adding URL node, skipping ... */
                 ERRVPRINT(VERBOSE1, "%s %s\n", _("Error adding URL node, skipping"), t_unode.string);
             }
@@ -796,3 +821,10 @@ restore_state()
     check_dup = 1;                              /* enable duplicate checking */
     return 0;                                   /* return with ok code       */
 }
+
+
+/************************************************************************
+ ************************************************************************
+ *                      END OF FILE                                     *
+ ************************************************************************
+ ************************************************************************/
